// footnotes
// v1.0

(function ($) {

  // custom select class

  function footnotes(item, options) {

    this.item = $(item);

    // options

    var defaults = {

      debug: true,
      selectors: {
        footnote: '.footnote a'
      },
      elements: {
        tooltip: null,
        clicked: null
      }
    };

    this.options = $.extend(true, defaults, options);
    this.init();
  }

  footnotes.prototype = {

    // init

    init: function () {

      var plugin_instance = this
      var plugin_item = plugin_instance.item
      var plugin_settings = plugin_instance.options
      var plugin_selectors = plugin_settings.selectors
      var plugin_elements = plugin_settings.elements

      if (plugin_settings.debug == true) {
        console.log('footnotes', 'init')
      }

      //
      // ELEMENTS
      //

      plugin_item.addClass('footnotes-initialized');

      plugin_elements.tooltip = $('<div class="footnote-tooltip">').appendTo('body')

      //
      // ACTIONS
      //

      // CLICK FOOTNOTE

      $('.footnote a').click(function(e) {

        e.preventDefault();
        e.stopPropagation();

        plugin_elements.clicked = $(this).parent();

        var re_open = true;
        var open_timeout = 0;

        // if the tooltip element is already shown

        if ($('body').hasClass('tooltip-open')) {

          // if the current item was clicked
          if (plugin_elements.clicked.hasClass('current-tooltip')) {

            re_open = false

          } else {

            open_timeout = 250

          }

          plugin_instance.close_tooltip()

        }

        if (re_open == true) {

          setTimeout(function() {
            plugin_instance.open_tooltip()
          }, open_timeout)

        }

      });

      // CLICK ANYWHERE TO CLOSE

      $('body').click(function() {

        plugin_instance.close_tooltip()

      });

      // CLICK FOOTNOTE

      $('.footnote-list-collapse').on('show.bs.collapse', function () {
        $(this).siblings('.footnote-list-head').addClass('footnote-list-open')
      })

      $('.footnote-list-collapse').on('hide.bs.collapse', function () {
        $(this).siblings('.footnote-list-head').removeClass('footnote-list-open')
      })

      //
      // READY
      //

      if (plugin_settings.debug == true) {
        console.log('footnotes', 'ready')
      }

    },

    close_tooltip: function(fn_options) {

      var plugin_instance = this
      var plugin_item = this.item
      var plugin_settings = plugin_instance.options
      var plugin_elements = plugin_settings.elements

      // options

      var settings = $.extend(true, {

      }, fn_options)

      $('body').removeClass('tooltip-open');

      plugin_elements.tooltip.css('left', '').css('top', '').removeClass('flipped');

      $('.footnote').removeClass('current-tooltip');

    },

    open_tooltip: function(fn_options) {

      var plugin_instance = this
      var plugin_item = this.item
      var plugin_settings = plugin_instance.options
      var plugin_elements = plugin_settings.elements

      // options

      var settings = $.extend(true, {

      }, fn_options)

      // populate
      plugin_instance.populate_tooltip();

      // show
      $('body').addClass('tooltip-open');

      // position
      plugin_instance.position_tooltip();

    },

    position_tooltip: function(fn_options) {

      var plugin_instance = this
      var plugin_item = this.item
      var plugin_settings = plugin_instance.options
      var plugin_elements = plugin_settings.elements

      // options

      var settings = $.extend(true, {

      }, fn_options)

      plugin_elements.tooltip.position({
        my: 'center top',
        at: 'center bottom',
        of: plugin_elements.clicked,
        using: function (obj,info) {
          console.log(obj);
          console.log(info);

          if (info.vertical != "top") {
            $(this).addClass("flipped");
          } else {
            $(this).removeClass("flipped");
          }

          $(this).css({
            left: obj.left + 'px',
            top: obj.top + 'px'
          });
        }
      });

    },

    populate_tooltip: function(fn_options) {

      var plugin_instance = this
      var plugin_item = this.item
      var plugin_settings = plugin_instance.options
      var plugin_elements = plugin_settings.elements

      // options

      var settings = $.extend(true, {

      }, fn_options)

      var fn_type = plugin_elements.clicked.attr('data-footnote-type')
      var fn_key = plugin_elements.clicked.attr('data-fn')

      plugin_elements.clicked.addClass('current-tooltip');

      var fn_content = $('.footnote-list[data-footnote-type="' + fn_type + '"] li[data-fn="' + fn_key + '"]').html();

      plugin_elements.tooltip.html('<div class="footnote-tooltip-content">' + fn_content + '</div>');

    },

    hide: function(fn_options) {

      var plugin_instance = this
      var plugin_item = this.item
      var plugin_settings = plugin_instance.options
      var plugin_elements = plugin_settings.elements

      // options

      var settings = $.extend(true, {

      }, fn_options)



    }

  }

  // jQuery plugin interface

  $.fn.footnotes = function (opt) {
    var args = Array.prototype.slice.call(arguments, 1);

    return this.each(function () {

      var item = $(this);
      var instance = item.data('footnotes');

      if (!instance) {

        // create plugin instance if not created
        item.data('footnotes', new footnotes(this, opt));

      } else {

        // otherwise check arguments for method call
        if (typeof opt === 'string') {
          instance[opt].apply(instance, args);
        }

      }
    });
  }

}(jQuery));




/*


(function($) {

  $(function() {

    var fn_tooltip = $('<div class="footnote-tooltip">').appendTo('body');

    function close_tooltip() {

      $('body').removeClass('tooltip-open');

      fn_tooltip.css('left', '').css('top', '').removeClass('flipped');

      $('.footnote').removeClass('current-tooltip');

    }

    function open_tooltip(fn_item) {

      // populate
      populate_tooltip(fn_item);

      // show
      $('body').addClass('tooltip-open');

      // position
      position_tooltip(fn_item);

    }

    function position_tooltip(fn_item) {

      fn_tooltip.position({
        my: 'center top',
        at: 'center bottom',
        of: fn_item,
        using: function (obj,info) {
          console.log(obj);
          console.log(info);

          if (info.vertical != "top") {
            $(this).addClass("flipped");
          } else {
            $(this).removeClass("flipped");
          }

          $(this).css({
            left: obj.left + 'px',
            top: obj.top + 'px'
          });
        }
      });

    }

    function populate_tooltip(fn_item) {

      var fn_type = fn_item.attr('data-type');
      var fn_key = fn_item.attr('data-fn');

      fn_item.addClass('current-tooltip');

      var fn_content = $('.footnote-list[data-type="' + fn_type + '"] li[data-fn="' + fn_key + '"]').html();

      fn_tooltip.html('<div class="footnote-tooltip-content">' + fn_content + '</div>');

    }

    $('.footnote a').click(function(e) {

      e.preventDefault();
      e.stopPropagation();

      var fn_item = $(this).parent();

      var re_open = true;
      var open_timeout = 0;

      // if the tooltip element is already shown

      if ($('body').hasClass('tooltip-open')) {

        // if the current item was clicked
        if (fn_item.hasClass('current-tooltip')) {

          re_open = false;

        } else {

          open_timeout = 250;

        }

        close_tooltip();

      }

      if (re_open == true) {

        setTimeout(function() {
          plugin_instance.open_tooltip(fn_item);
        }, open_timeout);

      }

    });

    $('body').click(function() {

      close_tooltip();

    });

  });
})(jQuery);


*/
