<?php

//
// OPTIONS PAGE
//

function footnotes_register_settings() {

  register_setting ( 'footnotes_options_group', 'footnotes_types', 'footnotes_callback' );

  register_setting ( 'footnotes_options_group', 'footnotes_logged_in', 'footnotes_callback' );

}

add_action ( 'admin_init', 'footnotes_register_settings' );

function footnotes_register_options_page() {

  add_options_page ( 'Footnote Settings', 'Footnotes', 'manage_options', 'footnotes', 'footnotes_options_page' );

}

add_action ( 'admin_menu', 'footnotes_register_options_page' );

function footnotes_options_page() {

?>

  <div>

    <h2>Footnote Settings</h2>

    <form method="post" action="options.php">

      <?php settings_fields ( 'footnotes_options_group' ); ?>

      <table>
        <tr valign="top">
          <th scope="row">
            <label for="footnotes_types">Enable references on post types</label>
          </th>

          <td>
						<textarea id="footnotes_types" name="footnotes_types"><?php echo implode ( "\n", get_option ( 'footnotes_types' ) ); ?></textarea>
          </td>

        </tr>
      </table>

      <?php submit_button(); ?>

    </form>
  </div>

<?php

}

//

function sanitize_types_val ( $new_value, $old_value ) {

	$new_value = preg_split ( '/\r\n?|\n/', $new_value );
	// $new_value = explode ( "\n", $new_value );

	return $new_value;

}

add_filter ( 'pre_update_option_footnotes_types', 'sanitize_types_val', 10, 2 );

//
// POST TYPES
//

// DEFINITION

function posttype_definition() {

	$labels = array(
		'name'                  => _x( 'Definitions', 'Post Type General Name', 'h7' ),
		'singular_name'         => _x( 'Definition', 'Post Type Singular Name', 'h7' ),
		'menu_name'             => __( 'Definitions', 'h7' ),
		'name_admin_bar'        => __( 'Definition', 'h7' ),
		'archives'              => __( 'Definition Archives', 'h7' ),
		'attributes'            => __( 'Definition Attributes', 'h7' ),
		'parent_item_colon'     => __( 'Parent Definition:', 'h7' ),
		'all_items'             => __( 'All Definitions', 'h7' ),
		'add_new_item'          => __( 'Add New Definition', 'h7' ),
		'add_new'               => __( 'Add New', 'h7' ),
		'new_item'              => __( 'New Definition', 'h7' ),
		'edit_item'             => __( 'Edit Definition', 'h7' ),
		'update_item'           => __( 'Update Definition', 'h7' ),
		'view_item'             => __( 'View Definition', 'h7' ),
		'view_items'            => __( 'View Definitions', 'h7' ),
		'search_items'          => __( 'Search Definition', 'h7' ),
		'not_found'             => __( 'Not found', 'h7' ),
		'not_found_in_trash'    => __( 'Not found in Trash', 'h7' ),
		'featured_image'        => __( 'Featured Image', 'h7' ),
		'set_featured_image'    => __( 'Set featured image', 'h7' ),
		'remove_featured_image' => __( 'Remove featured image', 'h7' ),
		'use_featured_image'    => __( 'Use as featured image', 'h7' ),
		'insert_into_item'      => __( 'Insert into item', 'h7' ),
		'uploaded_to_this_item' => __( 'Uploaded to this item', 'h7' ),
		'items_list'            => __( 'Items list', 'h7' ),
		'items_list_navigation' => __( 'Items list navigation', 'h7' ),
		'filter_items_list'     => __( 'Filter items list', 'h7' ),
	);
	$args = array(
		'label'                 => __( 'Definition', 'h7' ),
		'description'           => __( 'Post Type Description', 'h7' ),
		'labels'                => $labels,
		'supports'              => array( 'title', 'editor' ),
		'hierarchical'          => false,
		'public'                => true,
		'show_ui'               => true,
		'show_in_menu'          => true,
		'menu_position'         => 20,
		'menu_icon'             => 'dashicons-editor-help',
		'show_in_admin_bar'     => true,
		'show_in_nav_menus'     => true,
		'can_export'            => true,
		'has_archive'           => false,
		'exclude_from_search'   => true,
		'publicly_queryable'    => true,
		'capability_type'       => 'page',
	);
	register_post_type( 'definition', $args );

}

add_action( 'init', 'posttype_definition', 0 );

//
// SHORTCODES
//

// shortcode

function shortcode_footnote ( $atts ) {

	extract ( shortcode_atts ( array (
    'number' => '',
  ), $atts ) );

	$output = '<sup id="fnref:' . $number . '" class="footnote" data-footnote-type="footnote" data-fn="' . $number . '">';
	$output .= '<a id="footnote-' . $number . '" href="#fn:' . $number . '" rel="footnote">' . $number . '</a>';
  $output .= '</sup>';

	return $output;
}

add_shortcode ( 'footnote', 'shortcode_footnote' );

// reference

function shortcode_reference ( $atts ) {

	extract ( shortcode_atts ( array (
		'number' => '',
  ), $atts ) );

	$output = '<sup id="fnref:r' . $number . '" class="footnote" data-footnote-type="reference" data-fn="' . $number . '">';
	$output .= '<a id="reference-' . $number . '" href="#fn:r' . $number . '" rel="footnote">' . $number . '</a>';
  $output .= '</sup>';

	return $output;
}

add_shortcode ( 'reference', 'shortcode_reference' );

// define

function shortcode_define ( $atts, $content = null ) {

  global $definitions;

	$a = extract ( shortcode_atts ( array (
		'term' => ''
  ), $atts ) );

  if ( isset ( $term ) && $term != '' ) {

    $def_slug = $term;

  } else {

    $def_slug = $content;

    $def_slug = str_replace ( ' ', '-', strtolower ( $def_slug ) );

  }

  $definition_post = get_posts ( array (
    'post_type' => 'definition',
    'name' => $def_slug,
    'posts_per_page' => 1
  ) );

  if ( !empty ( $definition_post ) ) {

    $definition_post = $definition_post[0];

    $definition_slug = get_the_slug ( $definition_post->ID );

    $definitions[$definition_slug] = $definition_post->ID;

    $def_index = count ( $definitions );

  	$output = '<span id="fnref:d' . $def_index . '" class="footnote" data-footnote-type="definition" data-fn="' . $def_slug . '">';
  	$output .= '<a id="definition-' . $def_slug . '" href="#fn:d' . $def_index . '" rel="footnote">' . $content . '</a>';
    $output .= '</span>';

  } else {
    $output = $content;
  }

	return $output;
}

add_shortcode ( 'define', 'shortcode_define' );

//
// ACF
//

function footnotes_add_acf_fields() {

  if ( function_exists ( 'acf_add_local_field_group' ) ) :

		$field_group = array(
    	'key' => 'group_footnotes',
    	'title' => 'Footnotes & References',
    	'fields' => array(
    		array(
    			'key' => 'field_tab_footnotes',
    			'label' => 'Footnotes',
    			'name' => '',
    			'type' => 'tab',
    			'instructions' => '',
    			'required' => 0,
    			'conditional_logic' => 0,
    			'wrapper' => array(
    				'width' => '',
    				'class' => '',
    				'id' => '',
    			),
    			'placement' => 'top',
    			'endpoint' => 0,
    		),
    		array(
    			'key' => 'field_content_footnotes',
    			'label' => 'Footnotes',
    			'name' => 'page_footnotes',
    			'type' => 'wysiwyg',
    			'instructions' => '',
    			'required' => 0,
    			'conditional_logic' => 0,
    			'wrapper' => array(
    				'width' => '',
    				'class' => '',
    				'id' => '',
    			),
    			'default_value' => '',
    			'tabs' => 'all',
    			'toolbar' => 'full',
    			'media_upload' => 0,
    			'delay' => 0,
    		),
    		array(
    			'key' => 'field_tab_references',
    			'label' => 'References',
    			'name' => '',
    			'type' => 'tab',
    			'instructions' => '',
    			'required' => 0,
    			'conditional_logic' => 0,
    			'wrapper' => array(
    				'width' => '',
    				'class' => '',
    				'id' => '',
    			),
    			'placement' => 'top',
    			'endpoint' => 0,
    		),
    		array(
    			'key' => 'field_content_references',
    			'label' => 'References',
    			'name' => 'page_references',
    			'type' => 'wysiwyg',
    			'instructions' => '',
    			'required' => 0,
    			'conditional_logic' => 0,
    			'wrapper' => array(
    				'width' => '',
    				'class' => '',
    				'id' => '',
    			),
    			'default_value' => '',
    			'tabs' => 'all',
    			'toolbar' => 'full',
    			'media_upload' => 0,
    			'delay' => 0,
    		),
    	),
    	'location' => array(
    		array(
    			array(
    				'param' => 'post_type',
    				'operator' => '==',
    				'value' => 'page',
    			),
    		)
    	),
    	'menu_order' => 38,
    	'position' => 'normal',
    	'style' => 'default',
    	'label_placement' => 'top',
    	'instruction_placement' => 'label',
    	'hide_on_screen' => '',
    	'active' => true,
    	'description' => '',
    );

		// print_r($field_group);
		// echo '<hr>';

		$footnotes_types = get_option ( 'footnotes_types' );

		// print_r(json_encode($footnotes_types));

    if ( is_array ( $footnotes_types ) && !empty ( $footnotes_types ) ) {

      $field_group['location'] = array();

      foreach ( $footnotes_types as $post_type ) {

				$location = explode ( ',', $post_type );

        $field_group['location'][] = array (
          array (
            'param' => $location[0],
            'operator' => '==',
            'value' => $location[1],
          )
        );

      }

      // if ( !empty ( $field_locations ) ) {
      //   $field_group['location'] = $field_locations;
      // }

    }

				// print_r($field_group);
				// echo '<hr>';


    acf_add_local_field_group ( $field_group );

  endif;

}

add_action ( 'acf/init', 'footnotes_add_acf_fields' );
