<?php

# session_start();

//
// GLOBALS
//

function child_global_vars()
{

    global $vars;

    $query_string = $_GET;

    $vars['user_id'] = '';

    if (is_user_logged_in()) {
        $vars['user_id'] = get_current_user_id();
    }

    //
    // SYSTEM TAXONOMIES
    //

    $map_page = get_page_by_path('map');

    $row_content = get_field('map_rows', $map_page->ID);

    $vars['taxonomies'] = array(
        'influencer' => array(
            'name' => 'Influences',
            'slug' => 'influencer',
            'description' => $row_content['influencer']
        ),
        'source' => array(
            'name' => 'Source',
            'slug' => 'source',
            'description' => $row_content['source']
        ),
        'production' => array(
            'name' => 'Production',
            'slug' => 'production',
            'description' => $row_content['production']
        ),
        'conversion' => array(
            'name' => 'Conversion',
            'slug' => 'conversion',
            'description' => $row_content['conversion']
        ),
        'form' => array(
            'name' => 'Form',
            'slug' => 'form',
            'description' => $row_content['form']
        ),
        'transport' => array(
            'name' => 'Transport',
            'slug' => 'transport',
            'description' => $row_content['transport']
        ),
        'distribution' => array(
            'name' => 'Distribution',
            'slug' => 'distribution',
            'description' => $row_content['distribution']
        ),
        'use' => array(
            'name' => 'Use',
            'slug' => 'use',
            'description' => $row_content['use']
        ),
        'other' => array(
            'name' => 'Other Influences',
            'slug' => 'other',
            'description' => $row_content['other']
        )
    );

}

add_action('wp', 'child_global_vars');

//
// ENQUEUE
//

function child_theme_enqueue()
{
    $theme_dir = get_bloginfo('template_directory') . '/';

    $child_theme_dir = get_stylesheet_directory_uri() . '/';
    $child_js_dir = $child_theme_dir . 'resources/js/';


    // STYLES
    wp_enqueue_style('child-style', $child_theme_dir . 'style.css', array('global-style'), NULL, 'all');

    // SCRIPTS
    wp_register_script('child-functions', $child_js_dir . 'child-functions.js', array('jquery', 'global-functions', 'rellax'), null, true);
    wp_enqueue_script('child-functions');
}

add_action('wp_enqueue_scripts', 'child_theme_enqueue', 50);

//
// INCLUDES
//

$includes = array(
    'resources/functions/taxonomies.php',
    'resources/functions/post-types.php',
    'resources/functions/shortcodes.php'
);

foreach ($includes as $include) {

    if (locate_template($include) != '') {
        include_once(locate_template($include));
    }

}

//
// THEME SETUP
//

function custom_theme_setup()
{
    // DOCUMENT TITLE
    add_theme_support('title-tag');

    // PAGE EXCERPTS
    add_post_type_support('page', 'excerpt');

    // UNCROPPED IMAGE SIZE FOR CARD BLOCKS
    add_image_size('card-img-no-crop', '600', '380', false);
}

add_action('after_setup_theme', 'custom_theme_setup', 0);


//
// ADMIN STUFF
//

// DISABLE MEMBER ACCESS TO WP

function wpse66093_no_admin_access()
{
    // Do not run if the user is logged in and trying to log out
    // This might need one or two more checks.
    // Especially if you have custom login/logout/reset password/etc rules and routes set up.

    if (
        !is_admin() ||
        (
            is_user_logged_in() &&
            isset($GLOBALS['pagenow']) and 'wp-login.php' === $GLOBALS['pagenow']
        )
    ) {
        return;
    }

    $redirect = isset ($_SERVER['HTTP_REFERER']) ? $_SERVER['HTTP_REFERER'] : home_url('/');

    if (
        current_user_can('member') ||
        current_user_can('subscriber')
    ) {
        exit (wp_redirect($redirect));
    }

}

add_action('admin_init', 'wpse66093_no_admin_access', 100);

// ADD MEMBER USER

function xx__update_custom_roles()
{

    if (get_option('custom_roles_version') < 1) {

        add_role(
            'member',
            'Member',
            array(
                'read' => true,
                'level_0' => true
            )
        );

        update_option('custom_roles_version', 1);

    }

}

add_action('init', 'xx__update_custom_roles');

//
// FAVICON
//

function add_favicons()
{

    ?>

    <link rel="apple-touch-icon" sizes="180x180"
          href="<?php echo $GLOBALS['vars']['child_theme_dir']; ?>resources/img/favicon/apple-touch-icon.png">
    <link rel="icon" type="image/png" sizes="32x32"
          href="<?php echo $GLOBALS['vars']['child_theme_dir']; ?>/resources/img/favicon/favicon-32x32.png">
    <link rel="icon" type="image/png" sizes="16x16"
          href="<?php echo $GLOBALS['vars']['child_theme_dir']; ?>/resources/img/favicon/favicon-16x16.png">
    <link rel="manifest"
          href="<?php echo $GLOBALS['vars']['child_theme_dir']; ?>/resources/img/favicon/site.webmanifest">

    <?php

}

add_action('wp_head', 'add_favicons');

function add_og_tags()
{

    $og_title = '';
    $og_image = '';
    $og_description = '';

    if (is_single() || is_page()) {

        $og_title = get_the_title();
        $og_image = get_the_post_thumbnail_url(get_the_ID(), 'full');
        $og_description = get_the_excerpt(get_the_ID());

    } elseif (is_archive()) {

        $og_title = $GLOBALS['vars']['current_query']->name;
        $og_image = wp_get_attachment_image_url(get_field('term_image', $GLOBALS['vars']['current_query']->taxonomy . '_' . $GLOBALS['vars']['current_query']->term_id), 'full');
        $og_description = $GLOBALS['vars']['current_query']->description;

    }

    ?>

    <meta property="og:title" content="<?php echo $og_title; ?>"/>
    <meta property="og:image" content="<?php echo $og_image; ?>"/>
    <meta property="og:description" content="<?php echo $og_description; ?>"/>

    <?php

}

add_action('wp_head', 'add_og_tags');

function add_tag_manager1()
{

    ?>

    <!-- Google Tag Manager -->
    <script>(function (w, d, s, l, i) {
            w[l] = w[l] || [];
            w[l].push({
                'gtm.start':
                    new Date().getTime(), event: 'gtm.js'
            });
            var f = d.getElementsByTagName(s)[0],
                j = d.createElement(s), dl = l != 'dataLayer' ? '&l=' + l : '';
            j.async = true;
            j.src =
                'https://www.googletagmanager.com/gtm.js?id=' + i + dl;
            f.parentNode.insertBefore(j, f);
        })(window, document, 'script', 'dataLayer', 'GTM-WL6RRJ');</script>
    <!-- End Google Tag Manager -->

    <?php

}

add_action('wp_head', 'add_tag_manager1');

function add_tag_manager2()
{

    ?>

    <!-- Google Tag Manager (noscript) -->
    <noscript>
        <iframe src="https://www.googletagmanager.com/ns.html?id=GTM-WL6RRJ"
                height="0" width="0" style="display:none;visibility:hidden"></iframe>
    </noscript>
    <!-- End Google Tag Manager (noscript) -->

    <?php

}

add_action('wp_body_open', 'add_tag_manager2');

//
//
//

//
//
//

function acf_taxonomy_field($field)
{

    if (get_field('colour_options', 'option') != '') {

        $field['choices'] = array();

        $all_taxonomies = get_taxonomies(array(
            'public' => true,
        ), 'object');

        foreach ($all_taxonomies as $tax) {

            if ($tax->name != 'post_format') {
                $field['choices'][$tax->name] = $tax->label;
            }

        }

        return $field;

    }

}

// add to fields

$tax_fields = array(
    'taxonomy_list',
);

foreach ($tax_fields as $tax_field) {
    add_filter('acf/load_field/name=' . $tax_field, 'acf_taxonomy_field');
}


// filter

function custom_query_where($where)
{

    $where = str_replace("meta_key = 'event_dates_$", "meta_key LIKE 'event_dates_%", $where);

    return $where;

}

add_filter('posts_where', 'custom_query_where');


//
// TERM LIST
//


function output_term_list($terms, $reverse)
{

    $levels = count($terms);
    $current_level = 1;

    if (!empty ($terms)) {

        foreach ($terms as $tax => $terms) {

            ?>

            <div class="linked-term-list tax-<?php echo $tax; ?> level-<?php echo ($reverse == true) ? $levels - ($current_level - 1) : $current_level; ?>">
                <span class="bullet bg-tax-<?php echo $tax; ?>"></span>

                <span class="terms">
		<?php

        $i = 0;

        $max_terms = 2;

        foreach ($terms as $term) {

            if ($i <= ($max_terms - 1)) {

                $term_obj = get_term($term, $tax);

                if ($i > 0) echo ', ';

                echo $term_obj->name;

            }

            $i++;

        }

        ?>
	</span>

                <?php

                if (count($terms) > $max_terms) {
                    echo '<span class="plus rounded-pill">+' . (count($terms) - $max_terms) . '</span>';
                }

                ?>
            </div>

            <?php

            $current_level++;

        }

    }

}


//
// MAP FUNCTIONS
//

function map_get_topic_info()
{

    $term_data = get_term_by('id', $_POST['term_id'], $_POST['term_tax']);
    //
    // echo $term_data->taxonomy . '_' . $term_data->term_id;
    //
    // echo "\n" . get_field ( 'term_image', $term_data->taxonomy . '_' . $term_data->term_id );

    ?>

    <div class="node-info-content">
        <div class="node-head bg-dark">
            <?php

            if (get_field('term_image', $term_data->taxonomy . '_' . $term_data->term_id) != '') {

                ?>

                <div class="bg"
                     style="background-image: url(<?php echo wp_get_attachment_image_url(get_field('term_image', $term_data->taxonomy . '_' . $term_data->term_id), 'large'); ?>);"></div>

                <?php

            }

            ?>

            <h2 class="node-title"><?php echo $term_data->name; ?></h2>

            <a href="<?php echo esc_url(get_term_link($term_data)); ?>" class="node-head-link"></a>
        </div>

        <div class="node-desc">
            <!-- <h6>Description</h6> -->

            <?php echo apply_filters('the_content', $term_data->description); ?>

            <?php

            if (have_rows('tax_video', $term_data->taxonomy . '_' . $term_data->term_id)) {
                while (have_rows('tax_video', $term_data->taxonomy . '_' . $term_data->term_id)) {
                    the_row();

                    if (have_rows('videos')) {
                        while (have_rows('videos')) {
                            the_row();

                            ?>

                            <div class="video-container">
                                <?php

                                echo apply_filters('the_content', 'https://www.youtube.com/embed/' . get_sub_field('video'));

                                ?>
                            </div>


                            <?php

                        }
                    }

                }
            }

            ?>

        </div>
    </div>

    <a href="<?php echo esc_url(get_term_link($term_data)); ?>"
       class="btn btn-lg btn-secondary text-body has-icon icon-after">
        <span class="btn-text">View Topic</span>
        <i class="fas fa-long-arrow-alt-right text-white"></i>
    </a>

    <?php

    die();

}

add_action('wp_ajax_map_get_topic_info', 'map_get_topic_info');
add_action('wp_ajax_nopriv_map_get_topic_info', 'map_get_topic_info');

//
// GOOGLE ANALYTICS
//

function add_tracking_code()
{

    ?>

    <script async src="https://www.googletagmanager.com/gtag/js?id=UA-46156318-1"></script>
    <script> window.dataLayer = window.dataLayer || [];

        function gtag() {
            dataLayer.push(arguments);
        }

        gtag('js', new Date());
        gtag('config', 'UA-46156318-1'); </script>

    <?php

}

// add_action ( 'wp_footer', 'add_tracking_code' );

//
//
//

function get_author_info($author_ID = null)
{

    if ($author_ID == null || empty ($author_ID)) {
        $author_ID = get_the_author_meta('ID');
    }

    $author_data = array(
        'id' => $author_ID,
        'name' => get_user_meta($author_ID, 'first_name', true) . ' ' . get_user_meta($author_ID, 'last_name', true),
        'photo' => '',
        'title' => '',
        'org' => '',
        'url' => get_author_posts_url($author_ID),
        'show_chapter' => false,
        'show_archive' => false
    );

    //
    // CONTENT
    //

    if (have_rows('user_content', 'user_' . $author_ID)) {
        while (have_rows('user_content', 'user_' . $author_ID)) {
            the_row();

            if (get_sub_field('photo') != '') {
                $author_data['photo'] = get_sub_field('photo');
            }

            if (get_sub_field('title') != '') {
                $author_data['title'] = get_sub_field('title');
            }

        }
    }

    //
    // DISPLAY
    // show on chapter / show post archive
    //

    if (have_rows('user_display', 'user_' . $author_ID)) {
        while (have_rows('user_display', 'user_' . $author_ID)) {
            the_row();

            // chapter

            if (
                get_sub_field('chapter') == 1 &&
                get_field('user_chapter', 'user_' . $author_ID) != ''
            ) {

                $author_data['show_chapter'] = true;

            }

            // archive

            if (get_sub_field('archive') == 1) {
                $author_data['show_archive'] = true;
            }

        }
    }

    //
    // TYPE
    // staff or member
    //

    if (get_field('user_type', 'user_' . $author_ID) == 'staff') {

        // org

        $author_data['org'] = __('Student Energy', 'student-energy');

    } elseif (get_field('user_type', 'user_' . $author_ID) == 'member') {

        // org

        $chapter_ID = get_field('user_chapter', 'user_' . $author_ID);

        if ($chapter_ID != '') {
            $author_data['org'] = get_the_title($chapter_ID);
        }

        if ($author_data['show_chapter']) {
            $author_data['org'] = '<a href="' . get_permalink($chapter_ID) . '">' . get_the_title($chapter_ID) . '</a>';
        }

    }

    return $author_data;
}
