(function ($) {

  // custom select class

  function node_map(item, options) {

    this.item = $(item)

    // options

    var defaults = {
			container_pos: $('#map-container').offset(),
			nodes_sorted: [],
			subrow_heights: [100, 90, 80, 70, 60, 50, 40],
			node_margin_y: 10,
			node_margin_x: 50,
			interval_max: 50,
			interval_counter: 1,
			interval_step: 1,
			isolation_mode: true,
			current_node: null,
			current_links: null,
			current_linked_nodes: null,
      debug: false
    }

    this.options = $.extend(true, defaults, options)

    this.init()
  }

  node_map.prototype = {

    // init

    init: function () {

      var plugin_instance   = this
      var plugin_item       = plugin_instance.item
      var plugin_settings   = plugin_instance.options
      var plugin_classes    = plugin_settings.classes
      var plugin_elements   = plugin_settings.elements

      if (plugin_settings.debug == true) {
        console.log('node map', 'init');
      }

      //
      // ELEMENTS
      //

			//
			// INIT
			//

			$('#map-spinner').text('Initializing')
			$('body').addClass('spinner-on')

			$('.node').each(function() {
				$(this).attr('data-r', $(this).width())
			})

			// $('.link').each(function() {
			//
			// 	// $('.node[data-id=" + + "]')
			//
			// })

			plugin_instance.draw_links()

			//
			// ACTIONS
			//

			// window resize

			$(window).resize(function() {
				plugin_settings.container_pos = $('#map-container').offset()
				plugin_instance.draw_links()
			})

			// mode toggle

			$('#map-controls span').click(function() {

				$(this).siblings().removeClass('selected')
				$(this).addClass('selected')

				if ($('#map-controls .selected').attr('id') == 'map-mode-isolation') {
					plugin_settings.isolation_mode = true
					$('#map-container').addClass('isolation')
				} else {
					plugin_settings.isolation_mode = false
					$('#map-container').removeClass('isolation')
				}

			})

			// description

			$('.row-name').click(function() {

				if ($(this).closest('.node-row').hasClass('desc-open')) {
					$(this).closest('.node-row').removeClass('desc-open')
				} else {
					$(this).closest('.node-row').addClass('desc-open')

					if ($(window).width() < 576) {

						var clicked_pos = $(this).closest('.node-row').offset().top

						$('html, body').animate({
							scrollTop: clicked_pos
						}, {
							duration: 500
						})

					}
				}

				var line_interval = 0

				var interval_anim = setInterval(function() {

					if (line_interval >= 500) {

						clearInterval(interval_anim)

					} else {

						plugin_instance.draw_links()

						line_interval += 50

					}

				}, 10)

			})

			// node interactions

			$('.node-inner').on('mouseover', function(e) {

				if (!$('#map-container').hasClass('info-open')) {

					plugin_instance.draw_links()

					plugin_settings.current_node = $(this).closest('.node').attr('data-id')

					if (plugin_settings.current_node != null) {

						plugin_settings.current_linked_nodes = JSON.parse($(this).closest('.node').attr('data-targets'))

						plugin_instance.highlight_node()

					}

				}

			}).on('mouseout', function(e) {

				if (!$('#map-container').hasClass('info-open')) {

					plugin_settings.current_node = null
					plugin_settings.current_linked_nodes = null

					plugin_instance.reset_nodes()

				}

			}).on('click', function() {

				plugin_settings.current_node = $(this).closest('.node').attr('data-id')

				if (plugin_settings.current_node != null) {

					plugin_settings.current_linked_nodes = JSON.parse($(this).closest('.node').attr('data-targets'))

					plugin_instance.highlight_node()
					plugin_instance.open_info()

				}

			})

			$('#node-overlay').click(function() {

				plugin_instance.close_info()
				plugin_instance.reset_nodes()

			})

      //
      // ACTIONS
      //

			$('.node-info-inner').stick_in_parent({
				offset_top: sticky_offset
			})

			$('.row-name').stick_in_parent({
				offset_top: sticky_offset
			})

			$('body').removeClass('spinner-on')

      if (plugin_settings.debug == true) {
        console.log('node map', 'initialized')
      }

    },

		draw_links: function(fn_options) {

      var plugin_instance 	= this
      var plugin_item 			= this.item
      var plugin_settings 	= plugin_instance.options
      var plugin_classes 		= plugin_settings.classes
      var plugin_elements 	= plugin_settings.elements

      // options

      var settings = $.extend(true, {

      }, fn_options)

			plugin_settings.container_pos = $('#map-container').offset()

			$('.node').each(function(i) {

				var this_pos = $(this).offset()

				var this_x = this_pos.left + ($(this).outerWidth() / 2) - plugin_settings.container_pos.left,
						this_y = this_pos.top + ($(this).outerHeight() / 2) - plugin_settings.container_pos.top

				$(this).attr('data-x', this_x).attr('data-y', this_y)

				if ($(this).attr('data-id') == '116') {
					// console.log(this_x, this_y)
				}

				$('#map-svg .link[data-source="' + $(this).attr('data-id') + '"]')
					.attr('x1', this_x).attr('y1', this_y)

				$('#map-svg .link[data-target="' + $(this).attr('data-id') + '"]')
					.attr('x2', this_x).attr('y2', this_y)

			})

		},

		highlight_node: function(fn_options) {

      var plugin_instance 	= this
      var plugin_item 			= this.item
      var plugin_settings 	= plugin_instance.options
      var plugin_classes 		= plugin_settings.classes
      var plugin_elements 	= plugin_settings.elements

      // options

      var settings = $.extend(true, {
				selected_node: null,
				selected_links: null
      }, fn_options)

			// console.log(' - - - HIGHLIGHT - - - ')
			// console.log('selected', selected_node)
			// console.log('linked', selected_links)

			var node_element = $('.node[data-id="' + plugin_settings.current_node + '"]')

			// NODES

			$('.node').addClass('faded')

			// current
			node_element.removeClass('faded').addClass('hovered')

			// linked
			plugin_settings.current_linked_nodes.forEach(function(this_link) {
				$('.node[data-id="' + this_link + '"]').removeClass('faded').addClass('linked')
			})

			// LINKS

			$('#map-svg .link').each(function() {

				if (
					$(this).attr('data-target') == plugin_settings.current_node ||
					$(this).attr('data-source') == plugin_settings.current_node
				) {

					// linked
					$(this).removeClass('faded').addClass('linked')

				} else {

					// fade
					$(this).removeClass('linked').addClass('faded')

				}
			})

		},

		reset_nodes: function(fn_options) {

      var plugin_instance 	= this
      var plugin_item 			= this.item
      var plugin_settings 	= plugin_instance.options
      var plugin_classes 		= plugin_settings.classes
      var plugin_elements 	= plugin_settings.elements

      // options

      var settings = $.extend(true, {

      }, fn_options)

			// console.log(' - - - RESET - - - ')
			// console.log('selected', current_node)
			// console.log('linked', current_linked_nodes)

			$('#map-svg .link').removeClass('linked faded')
			$('.node').removeClass('hovered linked faded')

		},

		open_info: function(fn_options) {

      var plugin_instance 	= this
      var plugin_item 			= this.item
      var plugin_settings 	= plugin_instance.options
      var plugin_classes 		= plugin_settings.classes
      var plugin_elements 	= plugin_settings.elements

      // options

      var settings = $.extend(true, {
				selected_node: plugin_settings.current_node,
				selected_links: null
      }, fn_options)

			// console.log(' - - - LOAD - - - ')
			// console.log('selected', selected_node)
			// console.log('linked', selected_links)

			var node_element = $('.node[data-id="' + settings.selected_node + '"]')

			// spinner
			$('#map-spinner').text('Loading')
			$('body').addClass('spinner-on')

			// hide row descriptions
			$('.node-row').removeClass('desc-open')

			// add info open class
			$('#map-container').addClass('info-open transitioning')

			// do ajax
			plugin_instance.load_info()

			setTimeout(function() {

				$('html, body').animate({
					scrollTop: $('.node[data-id="' + settings.selected_node + '"]').offset().top - ($(window).height() / 2) + 40
				}, {
					duration: 500
				})

				$('#map-container').removeClass('transitioning')

				plugin_instance.draw_links()

			}, 1000)

		},

		load_info: function(fn_options) {

      var plugin_instance 	= this
      var plugin_item 			= this.item
      var plugin_settings 	= plugin_instance.options
      var plugin_classes 		= plugin_settings.classes
      var plugin_elements 	= plugin_settings.elements

      // options

      var settings = $.extend(true, {
				selected_node: null,
				selected_links: null
      }, fn_options)

			// console.log(' - - - LOAD - - - ')
			// console.log('selected', selected_node)
			// console.log('linked', selected_links)

			var node_element = $('.node[data-id="' + plugin_settings.current_node + '"]')

			$('#map-spinner').text('Fetching topic')
			$('body').addClass('spinner-on')

			$.ajax({
				type: 'POST',
				url: $('#map-container').attr('data-ajax-url'),
				data: {
					'term_id': plugin_settings.current_node,
					'term_tax': node_element.closest('.node-row').attr('data-tax'),
					'action': 'map_get_topic_info'
				},
				success: function (data) {

					$('#node-info').find('.node-info-inner').html(data).fadeIn(500, function() {

						setTimeout(function() {
							$(document.body).trigger('sticky_kit:recalc')
						}, 500)

						$('body').removeClass('spinner-on')
					})

				},
				error: function () {

					console.log('error')

				},
				complete: function() {

				}
			})

		},

		close_info: function(fn_options) {

      var plugin_instance 	= this
      var plugin_item 			= this.item
      var plugin_settings 	= plugin_instance.options
      var plugin_classes 		= plugin_settings.classes
      var plugin_elements 	= plugin_settings.elements

      // options

      var settings = $.extend(true, {
        selected_node: plugin_settings.current_node
      }, fn_options)

			// console.log(' - - - CLOSE - - - ')
			// console.log('selected', current_node)
			// console.log('linked', current_linked_nodes)

			// set classes
			$('#map-spinner').text('Loading map')
			$('body').addClass('spinner-on')
			$('#map-container').addClass('transitioning').removeClass('info-open')

			// remove info content
			$('#node-info').find('.node-info-inner').fadeOut(500, function() {
				$(this).empty()
			})

			setTimeout(function() {

				$('html, body').animate({
					scrollTop: $('.node[data-id="' + settings.selected_node + '"]').offset().top - ($(window).height() / 2) + 40
				}, {
					duration: 500,
					complete: function() {

						$('#map-container').removeClass('transitioning')
						plugin_instance.draw_links()

						$('body').removeClass('spinner-on')

					}
				})

			}, 750)

			plugin_settings.current_node = null
			plugin_settings.current_linked_nodes = null

		}

  }

  // jQuery plugin interface

  $.fn.node_map = function (opt) {

    var args = Array.prototype.slice.call(arguments, 1);

    return this.each(function () {

      var item = $(this);
      var instance = item.data('node_map');

      if (!instance) {

        // create plugin instance if not created
        item.data('node_map', new node_map(this, opt));

      } else {

        // otherwise check arguments for method call
        if (typeof opt === 'string') {
          instance[opt].apply(instance, args);
        }

      }
    });
  }

}(jQuery));



(function($) {
  jQuery(function($) {

		$(document).node_map()


  });
})(jQuery);
