(function($) {
    $(function() {

        var map_ID = 'chapter-map'

        var markers = JSON.parse($('#' + map_ID + '-data').attr('data-markers'))

        var chapter_map = L.map(map_ID + '-map', {
            center: [30, 0],
            zoom: 2,
            zoomControl: false,
            maxBounds: [[-90,-180], [90,180]]
        })

        //
        // ZOOM
        //

        new L.Control.Zoom({
            position: 'bottomleft'
        }).addTo(chapter_map)

        //
        // BASEMAP
        //

        L.tileLayer('https://api.mapbox.com/styles/v1/{id}/tiles/{z}/{x}/{y}?access_token=pk.eyJ1Ijoic2UtY29tbXVuaWNhdGlvbnMiLCJhIjoiY2xraHgzbW1xMDRuYjNqdDg0MGVzMXRmeiJ9.sG1SQ6nMLwkXtFANYMFhqA', {
            minZoom: 2,
            maxZoom: 18,
            attribution: 'Map data &copy; <a href="https://www.openstreetmap.org/">OpenStreetMap</a> contributors, ' +
                '<a href="https://creativecommons.org/licenses/by-sa/2.0/">CC-BY-SA</a>, ' +
                'Imagery © <a href="https://www.mapbox.com/">Mapbox</a>',
            id: 'mapbox/light-v9',
            tileSize: 512,
            zoomOffset: -1
        }).addTo(chapter_map)

        //
        // MARKERS
        //

        var chapter_marker = L.icon({
            iconUrl: $(document).data('child_theme_dir') + 'resources/img/marker.png',
            iconSize: [20, 30],
            iconAnchor: [10, 30],
            popupAnchor: [0, -40]
        })

        var chapter_marker_on = L.icon({
            iconUrl: $(document).data('child_theme_dir') + 'resources/img/marker_on.png',
            iconSize: [20, 30],
            iconAnchor: [10, 30],
            popupAnchor: [0, -40]
        })

        var old_layer = ''

        markers.forEach(function(marker, i) {

            markers[i] = L.marker(marker['coords'], { icon: chapter_marker })
                .bindPopup('<h4 class="text-white">' + marker['title'] + '</h4><p class="text-right"><a href="' + marker['url'] + '" class="btn btn-sm btn-secondary has-icon icon-after"><span class="btn-text">View Chapter</span><span class="btn-icon"><i class="fas fa-long-arrow-alt-right"></i></a></p>', { className: 'chapter-popup' })
                .on('click', function(e) {

                    e.target.setIcon(chapter_marker_on)

                    if (old_layer) {
                        old_layer.setIcon(chapter_marker)
                    }

                    old_layer = e.target

                    chapter_map.setView(e.target.getLatLng())

                })
                .addTo(chapter_map)

            markers[i].getPopup().on('remove', function(e) {

                old_layer.setIcon(chapter_marker)

            })

        })

        //
        // OVERLAY
        //

        chapter_map_overlay = $('<div class="map-overlay"><h5>Click to interact</h5></div>').appendTo('#' + map_ID + '-container')

        chapter_map_overlay.on('click', function(e) {
            $(this).fadeOut(125)
        })

        inView('#' + map_ID)
            .on('enter', function() {

                chapter_map.invalidateSize(false)

            }).on('exit', function() {

            chapter_map_overlay.show()
            // console.log('exit')

        })

    });
})(jQuery);