(function ($) {
    jQuery(function ($) {

        const map_ID = 'event-location-map';

        const markers = JSON.parse($('#' + map_ID + '-data').attr('data-markers'));

        const event_map = L.map(map_ID + '-map', {
            center: [30, 0],
            zoom: 2,

            maxBounds: [[-90, -180], [90, 180]]
        });

        //
        // ZOOM
        //


        //
        // BASEMAP
        //
        L.tileLayer('https://api.mapbox.com/styles/v1/{id}/tiles/{z}/{x}/{y}?access_token=pk.eyJ1IjoibWFwYm94IiwiYSI6ImNpejY4NXVycTA2emYycXBndHRqcmZ3N3gifQ.rJcFIG214AriISLbB6B5aw', {
            minZoom: 2,
            maxZoom: 18,
            attribution: 'Map data &copy; <a href="https://www.openstreetmap.org/">OpenStreetMap</a> contributors, ' +
                '<a href="https://creativecommons.org/licenses/by-sa/2.0/">CC-BY-SA</a>, ' +
                'Imagery © <a href="https://www.mapbox.com/">Mapbox</a>',
            id: 'mapbox/light-v9',
            tileSize: 512,
            zoomOffset: -1
        }).addTo(event_map)

        //
        // MARKERS
        //

        var chapter_marker = L.icon({
            iconUrl: $(document).data('child_theme_dir') + 'resources/img/marker.png',
            iconSize: [20, 30],
            iconAnchor: [10, 30],
            popupAnchor: [0, -40]
        })

        var chapter_marker_on = L.icon({
            iconUrl: $(document).data('child_theme_dir') + 'resources/img/marker_on.png',
            iconSize: [20, 30],
            iconAnchor: [10, 30],
            popupAnchor: [0, -40]
        })

        var old_layer = ''

        markers.forEach(function (marker, i) {

            markers[i] = L.marker([marker['lat'], marker['lng']], {icon: chapter_marker})
                .bindPopup('<p class="text-white">' + marker['label'] + '</p>', { className: 'chapter-popup'} )
                .on('click', (e) => {
                    e.target.setIcon(chapter_marker_on)

                    if (old_layer) {
                        old_layer.setIcon(chapter_marker)
                    }

                    old_layer = e.target

                    event_map.setView(e.target.getLatLng())

                })
                .addTo(event_map)

            markers[i].getPopup().on('remove', function (e) {
                old_layer.setIcon(chapter_marker)
            })
        })

        const group = new L.featureGroup(markers);

        event_map.fitBounds(group.getBounds());
    });
})(jQuery);
