/**
 * called by map_with_filter.php to set admin-d.php URL
 * @param newAdminAjaxUrl string URL of admin-ajax.php
 */
export function setAdminAjaxURL(newAdminAjaxUrl) {
    adminAjaxUrl = newAdminAjaxUrl
}


const mapWithFilterCollection = document.getElementsByClassName("map-with-filter")
let adminAjaxUrl

for (let mapWithFilter of mapWithFilterCollection) {
    mapWithFilter.querySelectorAll(".query-filter-list");
    const queryFilters = mapWithFilter.querySelectorAll(".query-filter"),
        checkboxes = mapWithFilter.querySelectorAll("li"),
        queryFilterClear = mapWithFilter.querySelector(".query-filter-clear"),
        markersData = mapWithFilter.parentNode.querySelector(".d-none")

    let checkboxesGroups = []

    queryFilters.forEach(queryFilter => {
        checkboxesGroups.push(queryFilter.querySelectorAll("li"))
    })


    let postType = mapWithFilter.getAttribute("data-post-type")

    // filter mgmt
    if (queryFilterClear !== null) {
        // checkboxes mgmt
        for (let checkbox of checkboxes) {
            checkbox.addEventListener("click", _=> {
                if (checkbox != null) {
                    checkboxManager(checkbox, checkboxesGroups, markersData, postType)
                    toggleClearFilter(checkboxes, queryFilterClear)
                }
            })
        }

        // clear checkboxes button mgmt
        queryFilterClear.addEventListener("click", _=> {
            clearCheckboxes(checkboxes, checkboxesGroups, queryFilterClear, markersData)
            ajaxMapWithFilter(checkboxesGroups, markersData, postType)
        })
    }

    // map logic
    window.addEventListener('load',_=> {
        manageLeaflet(mapWithFilter.parentElement.id, markersData, postType)
    })
}

/**
 * Toggles a checkbox
 * @param checkboxesGroups HTMLCollection[] Array of all the checkboxes, grouped by taxonomy
 * @param checkboxLiElement HTMLLIElement the checkbox div
 * @param markersData HTMLDivElement div containing markers data
 * @param postType string the type of posts shown
 */
function checkboxManager(checkboxLiElement, checkboxesGroups, markersData, postType) {
    toggleCheckbox(checkboxLiElement, checkboxesGroups, markersData, postType)
}


/**
 * Disables all checkboxes + hides the "Clear" button
 * @param checkboxes NodeListOf<HTMLLIElement> List of checkboxes
 * @param checkboxesGroups HTMLCollection[] Array of all the checkboxes, grouped by taxonomy
 * @param clearFilterElement HTMLSpanElement the "Clear" button
 * @param markersData HTMLDivElement div containing markers data
 * @param postType string the type of posts shown
 */
function clearCheckboxes(checkboxes, checkboxesGroups, clearFilterElement, markersData, postType) {
    for (const checkbox of checkboxes) {
        toggleCheckbox(checkbox.closest("li"), checkboxesGroups, markersData, postType,  true)
    }

    toggleClearFilter(checkboxes, clearFilterElement, postType)
}

/**
 * Enables / disable a given checkbox
 * @param checkboxLiElement HTMLElement the checkbox itself
 * @param checkboxesGroups HTMLCollection[] Array of all the checkboxes, grouped by taxonomy
 * @param markersData HTMLDivElement div containing markers data
 * @param postType string the type of posts shown
 * @param force bool disable checkbox event if already disabled
 */
function toggleCheckbox(checkboxLiElement, checkboxesGroups, markersData, postType, force = false) {
    const icon = checkboxLiElement.querySelector(".icon")

    if (checkboxLiElement.classList.contains("selected") || force) {
        checkboxLiElement.classList.remove("selected")
        icon.classList.remove("fa-square-check")
        icon.classList.add("fa-square")
    } else if (!checkboxLiElement.classList.contains("selected") && !force ) {
        checkboxLiElement.classList.add("selected")
        icon.classList.add("fa-square-check")
        icon.classList.remove("fa-square")
    }

    // only sends ajax request if not forced
    if (!force) {
        ajaxMapWithFilter(checkboxesGroups, markersData, postType)
    }
}


/**
 * Manages AJAX to update the map's markers
 * @param checkboxesGroups HTMLCollection[] Array of all the checkboxes, grouped by taxonomy
 * @param markersData HTMLDivElement div containing markers and post-type data
 * @param postType string the type of posts shown
 */
function ajaxMapWithFilter(checkboxesGroups, markersData, postType) {
    // gets the list of fields first
    let filters = []

    for (const checkboxesGroup of checkboxesGroups) {
        let terms = []

        for (let checkbox of checkboxesGroup) {
            if (checkbox.classList.contains("selected")) {
                terms.push(checkbox.getAttribute("data-filter-value"))
            }
        }

        if (terms.length > 0) {
            filters.push({
                "taxonomy": checkboxesGroup[0].getAttribute("data-filter-key"),
                "terms": terms
            })
        }
    }

    jQuery.ajax({
        url: adminAjaxUrl,
        type: "POST",
        dataType: "json",
        data: {
            "action": "map_markers_query",
            "post_type": postType,
            "filters": filters
        },
        success: (response) => {
            if (response.success) {
                markersData.setAttribute("data-markers", JSON.stringify(response["data"]))
            }
        }
    })
}

/**
 * Shows / hide toggle button if any checkbox is toggled
 * @param checkboxes HTMLCollection
 * @param clearFilterElement HTMLDivElement
 * @param force bool forces disabling toggle button
 */
function toggleClearFilter(checkboxes, clearFilterElement, force = false) {
    let checkboxToggled = false

    for (const checkbox of checkboxes) {
        if (checkbox.classList.contains("selected")) {
            checkboxToggled = true
        }
    }

    if (!checkboxToggled || force) {
        clearFilterElement.classList.remove("display")
    } else {
        clearFilterElement.classList.add("display")
    }
}

/**
 * Manages the map logic
 * @param mapID string id of the main div
 * @param markersData HTMLDivElement div containing markers and post-type data
 * @param postType string the type of posts shown
 */
function manageLeaflet(mapID, markersData, postType) {
    const observer = new MutationObserver(() => {
        setMarkers(JSON.parse(markersData.getAttribute('data-markers')))
    });

    observer.observe(markersData, {
        attributes: true,
        attributeFilter: ['data-markers']
    })

    let leafletMap = L.map(mapID + '-map', {
        center: [30, 0],
        zoom: 2,
        zoomControl: false,
        duration: 1000,
        maxBounds: [[-90, -180], [90, 180]]
    })

    // ZOOM
    new L.Control.Zoom({
        position: 'bottomleft'
    }).addTo(leafletMap)

    // BASEMAP
    L.tileLayer('https://api.mapbox.com/styles/v1/{id}/tiles/{z}/{x}/{y}?access_token=pk.eyJ1Ijoic2UtY29tbXVuaWNhdGlvbnMiLCJhIjoiY2xraHgzbW1xMDRuYjNqdDg0MGVzMXRmeiJ9.sG1SQ6nMLwkXtFANYMFhqA', {
        minZoom: 2,
        maxZoom: 18,
        attribution: 'Map data &copy; <a href="https://www.openstreetmap.org/">OpenStreetMap</a> contributors, ' +
            '<a href="https://creativecommons.org/licenses/by-sa/2.0/">CC-BY-SA</a>, ' +
            'Imagery © <a href="https://www.mapbox.com/">Mapbox</a>',
        id: 'mapbox/light-v11',
        tileSize: 512,
        zoomOffset: -1
    }).addTo(leafletMap)

    // OVERLAY
    let mapOverlay = jQuery('<div class="map-overlay"><h5>Click to interact</h5></div>').appendTo('#' + mapID + '-container')

    mapOverlay.on('click', function (e) {
        jQuery(this).fadeOut(125)
    })

    inView('#' + mapID)
        .on('enter', function () { leafletMap.invalidateSize(false) })
        .on('exit', function () { mapOverlay.show() })


    setMarkers(JSON.parse(markersData.getAttribute('data-markers')))

    function setMarkers(markersJson) {
        let markers = []

        let chapter_marker = L.icon({
            iconUrl: jQuery(document).data('child_theme_dir') + 'resources/img/marker.png',
            iconSize: [20, 30],
            iconAnchor: [10, 30],
            popupAnchor: [0, -40]
        })

        let chapter_marker_on = L.icon({
            iconUrl: jQuery(document).data('child_theme_dir') + 'resources/img/marker_on.png',
            iconSize: [20, 30],
            iconAnchor: [10, 30],
            popupAnchor: [0, -40]
        })

        let old_layer = ''

        markersJson.forEach(function (marker, i) {
            markers[i] = L.marker(marker['coords'], { icon: chapter_marker })
                .bindPopup(`
                        <h4 class="text-white">${marker['title']}</h4>
                        <p class="text-right">
                            <a href="${marker['url']}" class="btn btn-sm btn-secondary has-icon icon-after">
                                <span class="btn-text">View ${postType}</span>
                                <span class="btn-icon"><i class="fas fa-long-arrow-alt-right"></i></span>
                            </a>
                        </p>`, {
                    className: 'chapter-popup'
                })
                .on('click', function (e) {
                    e.target.setIcon(chapter_marker_on)

                    if (old_layer) { old_layer.setIcon(chapter_marker) }

                    old_layer = e.target
                    leafletMap.setView(e.target.getLatLng())
                })


            markers[i].getPopup().on('remove', function (e) { old_layer.setIcon(chapter_marker) })
        })

        let markersFeatureGroup = new L.featureGroup(markers)

        // clears old markers on the map
        jQuery(".leaflet-marker-icon").remove()
        jQuery(".leaflet-popup").remove()

        markersFeatureGroup.addTo(leafletMap)
        let bounds = markersFeatureGroup.getBounds().pad(0.05)
        leafletMap.flyToBounds(bounds)
    }
}