// fallback value, for legacy reasons
let map_ID = "chapters-map"

export function setMapId(newMapId) {
    map_ID = newMapId
}

(function () {
    jQuery(function ($) {
        const markersDataDiv = document.getElementById(`${map_ID}-data`)
        
        let chapter_map,
            type = $(`#${map_ID}-data + .map-with-filter`).attr("data-post-type")

        if (type === undefined) {
            type = "chapter"
        }

        // observer, fires when data-markers is updated, which updates the markers on the map
        const observer = new MutationObserver(() => {
            setMarkers()
        });

        observer.observe(markersDataDiv, {
            attributes: true,
            attributeFilter: ['data-markers']
        });

        initMap()
        setMarkers()

        function initMap() {
            chapter_map = L.map(map_ID + '-map', {
                center: [30, 0],
                zoom: 2,
                zoomControl: false,
                duration: 1000,
                maxBounds: [[-90, -180], [90, 180]]
            })

            //
            // ZOOM
            //
            new L.Control.Zoom({
                position: 'bottomleft'
            }).addTo(chapter_map)

            //
            // BASEMAP
            //
            L.tileLayer('https://api.mapbox.com/styles/v1/{id}/tiles/{z}/{x}/{y}?access_token=pk.eyJ1Ijoic2UtY29tbXVuaWNhdGlvbnMiLCJhIjoiY2xraHgzbW1xMDRuYjNqdDg0MGVzMXRmeiJ9.sG1SQ6nMLwkXtFANYMFhqA', {
                minZoom: 2,
                maxZoom: 18,
                attribution: 'Map data &copy; <a href="https://www.openstreetmap.org/">OpenStreetMap</a> contributors, ' +
                    '<a href="https://creativecommons.org/licenses/by-sa/2.0/">CC-BY-SA</a>, ' +
                    'Imagery © <a href="https://www.mapbox.com/">Mapbox</a>',
                id: 'mapbox/light-v11',
                tileSize: 512,
                zoomOffset: -1
            }).addTo(chapter_map)

            //
            // OVERLAY
            //
            let chapter_map_overlay = $('<div class="map-overlay"><h5>Click to interact</h5></div>').appendTo('#' + map_ID + '-container')

            chapter_map_overlay.on('click', function (e) {
                $(this).fadeOut(125)
            })

            inView('#' + map_ID)
                .on('enter', function () { chapter_map.invalidateSize(false) })
                .on('exit', function () { chapter_map_overlay.show() })
        }

        function setMarkers() {
            const dataDiv = $('#' + map_ID + '-data')
            let markersJson = JSON.parse(dataDiv.attr('data-markers')),
                markers = [],
                markersLatLng = []

            let chapter_marker = L.icon({
                iconUrl: $(document).data('child_theme_dir') + 'resources/img/marker.png',
                iconSize: [20, 30],
                iconAnchor: [10, 30],
                popupAnchor: [0, -40]
            })

            let chapter_marker_on = L.icon({
                iconUrl: $(document).data('child_theme_dir') + 'resources/img/marker_on.png',
                iconSize: [20, 30],
                iconAnchor: [10, 30],
                popupAnchor: [0, -40]
            })

            let old_layer = ''

            markersJson.forEach(function (marker, i) {
                markers[i] = L.marker(marker['coords'], { icon: chapter_marker })
                    .bindPopup(`
                        <h4 class="text-white">${marker['title']}</h4>
                        <p class="text-right">
                            <a href="${marker['url']}" class="btn btn-sm btn-secondary has-icon icon-after">
                                <span class="btn-text">View ${type}</span>
                                <span class="btn-icon"><i class="fas fa-long-arrow-alt-right"></i></span>
                            </a>
                        </p>`, {
                        className: 'chapter-popup'
                    })
                    .on('click', function (e) {
                        e.target.setIcon(chapter_marker_on)

                        if (old_layer) { old_layer.setIcon(chapter_marker) }

                        old_layer = e.target
                        chapter_map.setView(e.target.getLatLng())
                    })


                markers[i].getPopup().on('remove', function (e) { old_layer.setIcon(chapter_marker) })
            })

            let markersFeatureGroup = new L.featureGroup(markers)

            // clears old markers on the map
            $(".leaflet-marker-icon").remove()
            $(".leaflet-popup").remove()

            markersFeatureGroup.addTo(chapter_map)
            let bounds = markersFeatureGroup.getBounds().pad(0.05)
            chapter_map.flyToBounds(bounds)
        }
    });
})(jQuery);
