<?php

require_once get_template_directory() . "/blocks/query/FiltersManager.php";
require_once get_template_directory() . "/types/FilterTermType.php";
require_once get_template_directory() . "/types/FilterType.php";

use blocks\query\FiltersManager;
use types\FilterTermType;
use types\FilterType;

$filters_manager = new FiltersManager();

$filters = array();

// managed by ACF field "Interactive — Map with filter"
$user_selected_categories = get_sub_field("categories");

// iterates through the categories used to filter
if ($user_selected_categories) {
    foreach ($user_selected_categories as $category) {
        if ($category["filtering_enabled"] == 1) {
            $category_terms = get_terms(array(
                'taxonomy'   => $category["name"],
                'hide_empty' => true,
            ));

            // iterate through each categories' items (with content)
            $terms = array();

            if (!isset($category_terms->errors)) {
                foreach ($category_terms as $term) {
                    $terms[] = new FilterTermType(
                        key: $category["name"],
                        value: $term->term_id,
                        name: $term->name,
                    );
                }

                $filters[] = new FilterType(
                    type: "tx",
                    by: $category["name"],
                    multi: "true",
                    terms: $terms
                );
            }
        }
    }
}

$map_type = get_sub_field('type');

wp_enqueue_style('leaflet');

if ($map_type === 'leaflet') {
    wp_enqueue_script('leaflet');
} elseif ($map_type == 'highmaps') {
    wp_enqueue_script('highmaps');
}

wp_enqueue_script('renderer');

$js_includes = array();

// resources/maps/*.js
// array_push($js_includes, "map");

if (locate_template('resources/maps/map-with-filter.php') != '') {
    include(locate_template('resources/maps/map-with-filter.php'));
}
?>

<div class="map-with-filter"
     data-post-type="<?php echo get_sub_field("post_type") ?>">
    <?php
        if (sizeof($filters) > 0) {
            $filters_manager->display(
                title: "Filter",
                filters: $filters
            );
        }
    ?>

    <div id="<?php echo get_current_element_ID() ?>-container" class="map-container">
        <div id="<?php echo get_current_element_ID() ?>-map"
             class="renderable"
             data-renderable-type="map">
        </div>
    </div>
</div>

<script type="module">
    import {setAdminAjaxURL} from "<?php echo get_stylesheet_directory_uri() ?>/resources/maps/map-with-filter.js"

    setAdminAjaxURL("<?php echo admin_url('admin-ajax.php'); ?>")
</script>