<?php

namespace blocks\query;

use WP_Query;

class QueryManager
{
    public function get_map_markers(string $post_type, array $filters = null): array {
        $field_name = "location";
        if ($post_type === "chapter") {
            $field_name = "chapter_location";
        }
        
        $tax_query = array();

        if ($filters !== null) {
            foreach ($filters as $filter) {
                $tax_query[] = array(
                    'taxonomy'  => $filter["taxonomy"],
                    'field'     => 'term_taxonomy_id',
                    'terms'     => $filter["terms"]
                );
            }

            $query = new WP_Query(array(
                "post_type" => $post_type,
                "posts_per_page" => -1,
                "post_status" => "publish",
                'tax_query' => array(
                    'relation'      => 'OR',
                    $tax_query
                )
            ));
        } else {
            $query = new WP_Query(array(
                "post_type" => $post_type,
                "posts_per_page" => -1,
                "post_status" => "publish"
            ));
        }



        if ($query->have_posts()) {
            $markers = array();

            while ($query->have_posts()) {
                $query->the_post();
                $location = get_field($field_name);

                if ($location != null && $location["markers"] != null) {
                    $markers[] = array(
                        "id" => get_the_ID(),
                        "title" => get_the_title(),
                        "url" => get_permalink(),
                        "coords" => array(
                            $location["markers"][0]["lat"],
                            $location["markers"][0]["lng"]
                        )
                    );
                }
            }
            return $markers;
        }

        return array();
    }
}
