<?php


//
// GLOBAL VARS
//

function var_template_include($t)
{
    $GLOBALS['vars']['current_template'] = basename($t);
    return $t;
}

add_filter('template_include', 'var_template_include', 1000);

function theme_global_vars(): void {
    global $vars;
    global $classes;
    global $css;
    global $ids;
    global $elements;
    global $defaults;

    //
    // INLINE STYLES
    //

    $css = '';

    //
    // PAGE ID & CLASSES
    //

    $ids['body'] = 'page';

    //$classes['body'] = array( 'animsition', 'spinner-on' );

    if (is_front_page()) {

        $ids['body'] = 'page-home';

        if (get_field('home_progress') == 1) {
            $classes['body'][] = 'has-progress';
        }
    } elseif (is_author()) {

        $ids['body'] = 'page-author';
    } elseif (is_archive()) {

        $ids['body'] = 'page-archive';
    } elseif (is_home()) {

        $ids['body'] = 'page-posts';
    } else {

        $ids['body'] = get_post_type() . '-' . get_the_slug();
    }

    //
    // ELEMENTS
    //

    $elements = array(
        'current' => null,
        'types' => array(
            'section' => array(
                'type' => 'section',
                'current_id' => '',
                'classes' => array('h7-section', 'h7-page-section'),
                'collapsible' => array(
                    'enabled' => false,
                    'breakpoint' => ''
                ),
                'is_open' => false
            ),

            'container' => array(
                'type' => 'container',
                'current_id' => '',
                'classes' => array('h7-container', 'container-fluid'),
                'collapsible' => array(
                    'enabled' => false,
                    'breakpoint' => ''
                ),
                'is_open' => false
            ),

            'column' => array(
                'type' => 'column',
                'current_id' => '',
                'classes' => array('h7-column'),
                'collapsible' => array(
                    'enabled' => false,
                    'breakpoint' => ''
                ),
                'is_open' => false
            ),

            'block' => array(
                'type' => 'block',
                'current_id' => '',
                'classes' => array('h7-block'),
                'collapsible' => array(
                    'enabled' => false,
                    'breakpoint' => ''
                ),
                'is_open' => false
            )
        ),
        'counters' => array(
            'section' => 1,
            'container' => 1,
            'column' => 1,
            'block' => 1
        ),
        'id' => array(),
        'defaults' => array()
    );

    // $page_elements = array();

    //
    // DEFAULTS
    //

    if (have_rows('section_defaults', 'option')) {
        $defaults['section'] = get_field('section_defaults', 'option');
    }

    if (have_rows('container_defaults', 'option')) {
        $defaults['container'] = get_field('container_defaults', 'option');
    }

    if (have_rows('column_defaults', 'option')) {
        $defaults['column'] = get_field('column_defaults', 'option');
    }

    if (have_rows('block_defaults', 'option')) {
        $defaults['block'] = get_field('block_defaults', 'option');
    }

    //
    // HEADER SETTINGS
    //

    $classes['header'] = array();

    if (have_rows('header_settings', 'option')) {
        while (have_rows('header_settings', 'option')) {
            the_row();

            $classes['body'][] = 'header-position-' . get_sub_field('header_position');
            $vars['header']['position'] = get_sub_field('header_position');

            switch (get_sub_field('header_position')) {

                case 'default':
                    $classes['header'][] = 'position-relative';
                    break;

                case 'absolute':
                    $classes['header'][] = 'position-absolute';
                    break;

                case 'fixed':
                    $classes['header'][] = 'fixed-top';
                    break;

                case 'sticky':

                    $classes['header'][] = 'sticky';

                    function enqueue_sticky()
                    {
                        wp_enqueue_script('sticky-kit');
                    }

                    add_action('wp_enqueue_scripts', 'enqueue_sticky');

                    break;
            }

            if (have_rows('background')) {
                while (have_rows('background')) {
                    the_row();

                    if (get_sub_field('colour') != '') {
                        $classes['header'][] = 'bg-' . get_sub_field('colour');
                    }

                    if (get_sub_field('opacity') != '') {
                        $opacity = get_sub_field('opacity');
                        if (get_sub_field('opacity') < 1) $opacity = $opacity * 100;
                        $classes['header'][] = 'bg-opacity-' . $opacity;
                    }
                }
            }

            if (get_sub_field('header_position') == 'sticky' && have_rows('background_sticky')) {
                while (have_rows('background_sticky')) {
                    the_row();

                    if (get_sub_field('colour') != '') {
                        $classes['header'][] = 'bg-sticky-' . get_sub_field('colour');
                    }

                    if (get_sub_field('opacity') != '') {
                        $opacity = get_sub_field('opacity');
                        if (get_sub_field('opacity') < 1) $opacity = $opacity * 100;
                        $classes['header'][] = 'bg-sticky-opacity-' . $opacity;
                    }
                }
            }
        }
    }

    //
    // FOOTER SETTINGS
    //

    $classes['footer'] = array();

    if (have_rows('footer_settings', 'option')) {
        while (have_rows('footer_settings', 'option')) {
            the_row();

            // colours

            if (have_rows('colours')) {
                while (have_rows('colours')) {
                    the_row();

                    if (
                        get_sub_field('text_colour') != '' &&
                        get_sub_field('text_colour') != 'inherit'
                    ) {

                        $GLOBALS['css'] .= "\n" . '#page-footer { color: var(--' . get_sub_field('text_colour') . '); }';
                    }

                    if (
                        get_sub_field('heading_colour') != '' &&
                        get_sub_field('heading_colour') != 'inherit'
                    ) {

                        $GLOBALS['css'] .= "\n" . '#page-footer h1, #page-footer h2, #page-footer h3, #page-footer h4, #page-footer h5, #page-footer h6 { color: var(--' . get_sub_field('heading_colour') . '); }';
                    }

                    if (
                        get_sub_field('link_colour') != ''  &&
                        get_sub_field('link_colour') != 'inherit'
                    ) {

                        $GLOBALS['css'] .= "\n" . '#page-footer a { color: var(--' . get_sub_field('link_colour') . '); }';
                    }
                }
            }

            // background

            if (have_rows('background')) {
                while (have_rows('background')) {
                    the_row();

                    if (get_sub_field('colour') != '') {
                        $classes['footer'][] = 'bg-' . get_sub_field('colour');
                    }
                }
            }
        }
    }

    $vars['indent'] = 0;

    //
    // DATE / TIME
    //

    $vars['timestamp'] = current_time('timestamp');
    $vars['date'] = date('Ymd', $vars['timestamp']);

    //
    // URLS
    //

    // CURRENT SITE URL

    $vars['site_url'] = get_bloginfo('url');

    if (substr($vars['site_url'], -1) != '/') $vars['site_url'] .= '/';

    // NETWORK SITE URL

    $vars['network_site_url'] = $vars['site_url']; // default (not multisite)

    if (is_multisite()) {
        $vars['network_site_url'] = network_site_url();
    }

    // CURRENT LOCATION URL AND SLUG

    $vars['current_url'] = current_URL();
    $vars['current_slug'] = get_the_slug();
    $vars['current_ancestors'] = get_ancestors(get_the_ID(), get_post_type());

    // LANGUAGE

    $current_lang = 'en';

    // $vars['current_lang'] = ICL_LANGUAGE_CODE;
    // $classes['body'][] = 'lang-' . $vars['current_lang'];

    //
    // HOME PAGE ID
    //

    $vars['homepage'] = get_option('page_on_front');

    //
    // CURRENT QUERY
    //

    // global $current_query;
    $vars['current_query'] = get_queried_object();

    //
    // DIRECTORIES
    //

    // PARENT

    $vars['theme_dir'] = get_bloginfo('template_directory') . '/';

    // CHILD

    $vars['child_theme_dir'] = $vars['theme_dir']; // default (not a child theme)

    if (is_child_theme()) {
        $vars['child_theme_dir'] = get_stylesheet_directory_uri() . '/';
    }

    //
    // USER
    //

    if (is_user_logged_in()) {
        if (current_user_can('administrator')) {
            $classes['body'][] = 'logged-in-admin';
        }
    }
}

add_action('wp', 'theme_global_vars');


//
// THEME OPTIONS
//

function theme_options(): void {

    global $vars;

    // SOCIAL ACCOUNTS

    if (have_rows('social_options', 'option')) {
        while (have_rows('social_options', 'option')) {
            the_row();

            // redirect URL for sharing inline content with hash

            $vars['social']['redirect_URL'] = get_permalink(get_sub_field('redirect'));

            // SITES

            $social_sites = array('facebook', 'twitter', 'instagram', 'linkedin', 'pinterest', 'youtube', 'vimeo');

            foreach ($social_sites as $site) {
                if (have_rows($site)) {
                    while (have_rows($site)) {
                        the_row();

                        switch ($site) {
                            case 'facebook':
                                $site_name = __('Facebook', 'h7-social-widget');
                                break;

                            case 'twitter':
                                $site_name = __('Twitter', 'h7-social-widget');
                                break;

                            case 'linkedin':
                                $site_name = __('LinkedIn', 'h7-social-widget');
                                break;

                            case 'instagram':
                                $site_name = __('Instagram', 'h7-social-widget');
                                break;

                            case 'pinterest':
                                $site_name = __('Pinterest', 'h7-social-widget');
                                break;

                            case 'youtube':
                                $site_name = __('YouTube', 'h7-social-widget');
                                break;

                            case 'vimeo':
                                $site_name = __('Vimeo', 'h7-social-widget');
                                break;

                            default:
                                $site_name = '';
                        }

                        $vars['social']['sites'][$site] = array(
                            'site' => $site,
                            'name' => $site_name,
                            'account' => get_sub_field('account'),
                            'icon' => get_sub_field('icon')
                        );

                        $widgets = get_sub_field('widgets');

                        if (is_array($widgets) && in_array('follow', $widgets)) {
                            $vars['social']['follow']['items'][] = $site;
                        }

                        if (is_array($widgets) && in_array('share', $widgets)) {
                            $vars['social']['share']['items'][] = $site;
                        }
                    }
                }
            }

            // WIDGET OPTIONS

            // follow

            while (have_rows('follow')) {
                the_row();

                $vars['social']['follow']['widget_icon'] = get_sub_field('icon');
                $vars['social']['follow']['widget_text'] = get_sub_field('text');
            }

            // share

            while (have_rows('share')) {
                the_row();

                $vars['social']['share']['widget_icon'] = get_sub_field('icon');
                $vars['social']['share']['widget_text'] = get_sub_field('text');
                $vars['social']['share']['sites'] = get_sub_field('sites');
            }
        }
    }

    // BACKGROUND GALLERY

    if (have_rows('hero_default', 'option')) {
        while (have_rows('hero_default', 'option')) {
            the_row();

            $vars['bgs'] = get_sub_field('backgrounds');
        }
    }
}

add_action('wp', 'theme_options');

//
// INCLUDES
//

$includes = array(
    'resources/functions/essentials/overrides.php',
    'resources/functions/essentials/blocks.php',
    'resources/functions/essentials/post.php',
    'resources/functions/essentials/taxonomy.php',
    'resources/functions/essentials/content.php',
    'resources/functions/essentials/layout.php',
    'resources/functions/essentials/media.php',
    'resources/functions/essentials/misc.php',
    'resources/functions/essentials/post-types.php',
    'resources/functions/essentials/taxonomies.php',
    'resources/functions/essentials/shortcodes.php',
    'resources/functions/acf/acf.php',
    'resources/functions/loop/elements.php',
    'resources/functions/loop/extras.php',
    'resources/functions/loop/utilities.php',
    'resources/functions/template/hero.php',
    'resources/functions/template/menu.php',
    'resources/functions/extensions/taxonomies.php',
    'resources/functions/extensions/post-types.php',
    'resources/functions/extensions/shortcodes.php'
);

foreach ($includes as $include) {

    if (locate_template($include) != '') {
        include_once(locate_template($include));
    }
}

//
// THEME FEATURES
//

function theme_features(): void {

    // menus

    add_theme_support('menus');

    register_nav_menus(array(
        'primary' => 'Primary Navigation',
        'secondary' => 'Secondary Navigation',
        'footer' => 'Footer Menu'
    ));

    // image sizes

    add_theme_support('post-thumbnails');

    add_image_size('bg', '2400', '1800', true);
    add_image_size('bg-no-crop', '2400', '1800', false);
    add_image_size('card-img', '600', '380', true);

    // ACF options page

    if (function_exists('acf_add_options_page') && current_user_can('administrator')) {

        acf_add_options_page(array(
            'page_title'     => 'Theme Settings',
            'menu_title'    => 'Theme Settings',
            'menu_slug'     => 'theme-settings',
            'capability'    => 'edit_posts'
        ));

        acf_add_options_sub_page(array(
            'page_title'  => 'Default Settings',
            'menu_title'  => 'Defaults',
            'parent_slug' => 'theme-settings',
        ));

        acf_add_options_sub_page(array(
            'page_title'  => 'Header',
            'menu_title'  => 'Header',
            'parent_slug' => 'theme-settings',
        ));

        acf_add_options_sub_page(array(
            'page_title'  => 'Footer',
            'menu_title'  => 'Footer',
            'parent_slug' => 'theme-settings',
        ));

        acf_add_options_sub_page(array(
            'page_title'  => 'Components',
            'menu_title'  => 'Component Settings',
            'parent_slug' => 'theme-settings',
        ));
    }
}

add_action('after_setup_theme', 'theme_features');

//
// ENQUEUE
//

// FRONT-END

function theme_enqueue(): void {
    //
    // VARS
    //

    $theme_dir = get_bloginfo('template_directory') . '/';
    $npm_dir = $theme_dir . 'resources/node_modules/';
    $phil_evans_dependencies_dir = $theme_dir . 'resources/phil_evans_dependencies/';
    $js_dir = $theme_dir . 'resources/js/';

    //
    // STYLES
    //

    wp_dequeue_style('wp-block-library');

    // global
    wp_register_style('global-style', $theme_dir . 'style.css', NULL, NULL, 'all');
    wp_enqueue_style('global-style');

    // user
    if (is_user_logged_in()) {
        wp_register_style('user-frontend', $theme_dir . 'resources/css/user/frontend.css', NULL, NULL, 'all');
        wp_enqueue_style('user-frontend');
    }

    //
    // SCRIPTS
    //

    // REGISTER
    wp_enqueue_script('jquery');


    // utilities
    wp_register_script('smooth-scroll', $phil_evans_dependencies_dir . 'pe-smooth-scroll/smooth-scroll.js', array('jquery'), NULL, true);

    //
    // components
    //

    // social widgets
    wp_register_script('share-widget', $phil_evans_dependencies_dir . 'pe-social-widget/share-widget.js', array('jquery'), NULL, true);
    wp_register_script('follow-widget', $phil_evans_dependencies_dir . 'pe-social-widget/follow-widget.js', array('jquery'), NULL, true);

    // supermenu
    wp_register_script('supermenu', $phil_evans_dependencies_dir . 'pe-supermenu/supermenu.js', array('jquery', 'bootstrap-js', 'slick'), NULL, true);

    // overlay
    wp_register_script('overlay', $phil_evans_dependencies_dir . 'pe-overlay/overlay.js', array('jquery'), NULL, true);


    //
    // renderables
    //

    // dependencies
    wp_register_script('scroll-progress', $phil_evans_dependencies_dir . 'pe-scroll-progress/scroll-progress.js', array('jquery'), NULL, true);

    wp_register_script('time-series-slider', $phil_evans_dependencies_dir . 'pe-time-series-slider/time-series-slider.js', array('jquery'), NULL, true);

    wp_register_script('pe-swiper', $phil_evans_dependencies_dir . 'pe-swiper/swiper.js', array('jquery'), NULL, true);

    wp_register_script('hotspots', $phil_evans_dependencies_dir . 'pe-hotspots/hotspots.js', array('jquery', 'jquery-ui-position'), NULL, true);

    // page functions
    wp_register_script('helpers', $js_dir . 'helper-functions.js', array('jquery'), NULL, true);
    wp_register_script('global-functions', $js_dir . 'global-functions.js', array('jquery', 'helpers', 'smooth-scroll'), NULL, true);

    wp_register_script('home-functions', $js_dir . 'home-functions.js', array('jquery'), NULL, true);

    // NPM dependencies
    wp_register_script('slick', $npm_dir . 'slick-carousel/slick/slick.min.js', array('jquery'), NULL, true);
    wp_register_script('sticky-kit', $npm_dir . 'sticky-kit/dist/sticky-kit.js', array('jquery'), NULL, true);

    wp_register_script('swiper', $npm_dir . 'swiper/swiper-bundle.min.js', NULL, NULL, true);

    wp_register_script('lazy', $npm_dir . 'jquery-lazy/jquery.lazy.min.js', array('jquery'), NULL, true);

    wp_register_script('rellax', $npm_dir . 'rellax/rellax.min.js', NULL, NULL, true);

    wp_register_script('magnify', $npm_dir . 'magnify/dist/js/jquery.magnify.js', NULL, NULL, true);

    wp_register_script('in-view', $npm_dir . 'in-view/dist/in-view.min.js', NULL, NULL, true);

    wp_register_script('isotope', $npm_dir . 'isotope-layout/dist/isotope.pkgd.min.js', array('jquery'), NULL, true);

    wp_register_script('select2', $npm_dir . 'select2/dist/js/select2.full.min.js', array('jquery'), NULL, true);

    wp_register_script('popper-utils', $npm_dir . '@popper.js/core/dist/umd/popper-utils.min.js', NULL, NULL, true);
    wp_register_script('popper', $npm_dir . 'popper.js/dist/umd/popper.min.js', NULL, NULL, true);
    
    wp_register_script('bootstrap-js', $npm_dir . 'bootstrap/dist/js/bootstrap.bundle.min.js', array('jquery'), NULL, true);

    wp_register_script('leaflet', $npm_dir . 'leaflet/dist/leaflet.js', NULL, NULL, true);
    wp_register_style('leaflet', $npm_dir . 'leaflet/dist/leaflet.css', NULL, NULL, 'all');

    wp_register_script('highmaps', $npm_dir . 'highcharts/highmaps.js', NULL, NULL, 'all');


    wp_register_script('highcharts', $npm_dir . 'highcharts/highcharts.js', NULL, NULL, true);
    wp_register_script('highcharts-more', $npm_dir . 'highcharts/highcharts-more.js', NULL, NULL, true);
    wp_register_script('highcharts-data', $npm_dir . 'highcharts/modules/data.js', NULL, NULL, true);
    wp_register_script('highcharts-exporting', $npm_dir . 'highcharts/modules/exporting.js', NULL, NULL, true);
    wp_register_script('highcharts-export-data', $npm_dir . 'highcharts/modules/export-data.js', NULL, NULL, true);

    // renderer
    wp_register_script('renderer', $js_dir . 'renderer.js', array('jquery'), NULL, true);

    // ENQUEUE

    // global
    wp_enqueue_script('bootstrap-js');
    wp_enqueue_script('sticky-kit');
    wp_enqueue_script('in-view');

    //wp_enqueue_script ( 'follow-widget' );
    //wp_enqueue_script ( 'share-widget' );
    //wp_enqueue_script ( 'smooth-scroll' );

    //wp_enqueue_script ( 'supermenu' );

    //
    // REQUIRED
    //

    wp_enqueue_script('overlay');
    wp_enqueue_script('global-functions');
}

add_action('wp_enqueue_scripts', 'theme_enqueue');

// ADMIN

// enqueue

function load_custom_wp_admin_style(): void {

    $theme_dir = get_bloginfo('template_directory') . '/';
    $npm_dir = $theme_dir . 'resources/node_modules/';
    $phil_evans_dependencies_dir = $theme_dir . 'resources/phil_evans_dependencies/';
    $js_dir = $theme_dir . 'resources/js/';

    //
    // CSS
    //

    wp_register_style('admin-style', $theme_dir . 'resources/css/user/admin.css', NULL, NULL, 'all');
    wp_enqueue_style('admin-style');

    //
    // JS
    //

    wp_register_script('hotspots-admin', $phil_evans_dependencies_dir . 'pe-hotspots/hotspots-admin.js', array('jquery', 'jquery-ui-position'), NULL, true);

    wp_register_script('admin', $js_dir . 'admin-functions.js', array('jquery'), NULL, true);

    wp_enqueue_script('admin');

    $admin_screen = get_current_screen();

    // print_r($admin_screen);

    if ($admin_screen->base == 'post') {
        wp_enqueue_script('hotspots-admin');
    }
}

add_action('admin_enqueue_scripts', 'load_custom_wp_admin_style');

//
// WPML
//

// get translated ID by path

function filtered_ID_by_path($path, $lang = null): void
{

    if ($lang == null) {
        if (isset($GLOBALS['vars']['current_lang'])) {
            $lang = $GLOBALS['vars']['current_lang'];
        } else {
            $lang = 'en';
        }
    }

    $get_page = get_page_by_path($path);

    if (!empty($get_page)) {
        apply_filters('wpml_object_id', $get_page->ID, 'page', true, $lang);
    }
}

//
//
//
// HOMELESS
//
//
//


//
// allow json uploads
//

add_filter('upload_mimes', 'h7_custom_mime_types', 1, 1);

function h7_custom_mime_types($mime_types)
{
    $mime_types['json'] = 'application/json';
    return $mime_types;
}


//
// LAYOUT UTILITIES
//



// 'settings' clones from the blocks flex field

$block_names = array(
    'heading' => '5d07f0390631a',
    'text' => '5d5d87e0ac22d',
    'jumbotron' => '5d261b80f4347',
    'buttons' => '5d5d4bcdcc84a'
);

// flipped array

$block_names_by_key = array_flip($block_names);

// default value sub-fields & keys

// fields
$field_names = array(
    'colours' => array(
        'btn_bg'          => '5d5d4d7deed1f',
        'btn_text'        => '5d5d4da2eed20',
        'btn_border'      => '5d5d4deeeed21'
    )
);

// clones

$clones = array(
    'colours' => array(
        'heading_colour'  => '5d26220eeb77c',
        'text_colour'     => '5d4d83a09c0f6',
        'link_colour'     => '5d4d86110e243',
    ),
    'background' => array(
        'colour' => '5ccb2eaa6a64d'
    )
);

function block_defaults_init(): void {
    global $block_names;
    global $block_names_by_key;
    global $field_names;
    global $clones;

    $admin_screen = get_current_screen();

    if ($admin_screen->base == 'post') {

        // fields
        foreach ($field_names as $field => $sub_fields) {
            foreach ($sub_fields as $sub_field => $field_key) {
                add_filter('acf/prepare_field/key=field_' . $field_key, 'get_field_defaults');
            }
        }

        // clones
        foreach ($block_names as $block => $block_key) {
            foreach ($clones as $field => $sub_fields) {
                foreach ($sub_fields as $sub_field => $field_key) {
                    // echo "\n" . $block . '/' . $field . '/' . $sub_field . ': field_' . $block_key . '_field_' . $field_key;

                    add_filter('acf/prepare_field/key=field_' . $block_key . '_field_' . $field_key, 'get_field_defaults');
                }
            }
        }
    }
}

add_action('admin_head', 'block_defaults_init');

function get_container_default_width($field)
{
    $width_default = '';

    if (have_rows('container_defaults', 'option')) {
        while (have_rows('container_defaults', 'option')) {
            the_row();

            if (get_sub_field('width') != '') {
                $width_default = get_sub_field('width');
            }
        }
    }

    $this_value = $field['value'];

    if ($width_default != '' && ($this_value == '' || empty($this_value))) {
        $this_value = $width_default;
        $field['value'] = $this_value;
    }

    return $field;
}

add_filter('acf/prepare_field/key=field_5dc0533eec190', 'get_container_default_width');


// add_filter ( 'acf/prepare_field/key=field_5d5d4d7deed1f', 'get_field_defaults' );

/*
function field_defaults_js() {

  $theme_dir = get_bloginfo ( 'template_directory' ) . '/';
  $npm_dir = $theme_dir . 'resources/node_modules/';
  $js_dir = $theme_dir . 'resources/js/';

  // wp_register_script ( 'block-defaults', $js_dir . 'block-defaults.js', array ( 'jquery' ), NULL, true );

  $admin_screen = get_current_screen();

  if ( $admin_screen->base == 'post' ) {
    // wp_enqueue_script ( 'hotspots-admin' );

    $defaults_array = array();

    $default_fields = array ( 'text', 'jumbotron', 'buttons' );

    foreach ( $default_fields as $field_name ) {

      $defaults_array[$field_name] = get_field ( $field_name . '_defaults', 'option' );

    }

?>

<script>

(function($) {
  jQuery(function($) {

    $(document).data('block_defaults', <?php echo json_encode ( $defaults_array ); ?>)

    console.log($(document).data('block_defaults'))

  });
})(jQuery);

</script>

<?php

  }

}

add_action ( 'admin_footer', 'field_defaults_js' );
*/

//
// FLEX FIELD 'NO VALUE' MESSAGES
//

add_filter('acf/fields/flexible_content/no_value_message', 'acf_flex_message', 10, 3);

function acf_flex_message($string = null, $field = null)
{

    switch ($field['key']) {

        // block - content
        case 'field_5dc4423f87c9b':
        case 'field_5dbc5d79be45e':
            $string = 'Click "Add Block" to select an element.';
            break;

        // block - interactive
        case 'field_5d2f237183591':
            $string = 'Click "Add Interactive" to select an element.';
            break;

        // block - navigation

        // settings
        case 'field_5dc1d75b83c2c':
            $string = 'Click "Add Setting" to select which option you want to override for this element.';
            break;
    }

    return $string;
}


// MAPS WITH FILTER
require_once get_template_directory() . "/blocks/query/QueryManager.php";
use blocks\query\QueryManager;

add_action( 'wp_ajax_map_markers_query', 'map_markers_query' );
add_action( 'wp_ajax_nopriv_map_markers_query', 'map_markers_query');

function map_markers_query(): void
{
    $query_manager = new QueryManager();

    $filters = null;
    $post_type = "";

    isset($_POST["filters"]) && $filters = $_POST["filters"];
    isset($_POST["post_type"]) && $post_type = $_POST["post_type"];

    $markers = $query_manager->get_map_markers(
        post_type: $post_type,
        filters: $filters
    );

    if ($markers != null) {
        wp_send_json_success($markers);
    } else {
        wp_send_json_error("no markers found");
    }
}
