<?php

//
// CONTENT LOOP
// this is the main recursive loop that cycles through the 'elements' flex field
// and sets up / outputs the elements
//

function content_loop ( $loop_key = null, $loop_ID = null, $resolve = true ) {

  // default values for the have_rows functions

  if ( !isset ( $loop_key ) ) $loop_key = 'elements';
  if ( !isset ( $loop_ID ) ) $loop_ID = get_the_ID();

  $GLOBALS['vars']['loop_key'] = $loop_key;
  $GLOBALS['vars']['loop_ID'] = $loop_ID;

  // start cycling through the 'elements' flex field
  // and setup each element

  if ( have_rows ( $loop_key, $loop_ID ) ) {

    $GLOBALS['vars']['loop_key'] = $loop_key;
    $GLOBALS['vars']['loop_ID'] = $loop_ID;

    while ( have_rows ( $loop_key, $loop_ID ) ) {
      the_row();

      //
      // LAYOUT TYPES
      // i.e. section, container, column, block
      //

      $this_layout = get_row_layout();

      if ( $this_layout == 'include' ) {

        // FILE INCLUDE

        // make sure the file exists

        if ( locate_template ( 'template/' . get_sub_field ( 'filename' ) . '.php' ) != '' ) {

          // close the previous element,
          // dump the contents of the provided file name and move on

					if ( isset ( $GLOBALS['elements']['current'] ) ) {
						close_element ( $GLOBALS['elements']['current']['type'] );
					}

          include ( locate_template ( 'template/' . get_sub_field ( 'filename' ) . '.php' ) );

        }

      } elseif ( $this_layout == 'template' ) {

        // TEMPLATE POST OBJECT

        // get the post ID selected in the template sub-field
        $template_ID = get_sub_field ( 'template' );

        $GLOBALS['vars']['loop_ID'] = $template_ID;

				// print_r($GLOBALS['elements']['current'] );

        // if the field layout is a template,
        // run content_loop recursively for the given template ID
        // if THAT template includes ANOTHER template,
        // the function will run again (and again and again...)

        content_loop ( 'elements', $template_ID );

      } else {

        $GLOBALS['vars']['loop_key'] = $loop_key;
        $GLOBALS['vars']['loop_ID'] = $loop_ID;

        // ANYTHING ELSE
        // i.e. a layout element with sub-fields, settings, etc.

        // resolve surrounding elements
        resolve_elements ( $this_layout, $loop_ID );

        // open this element
        open_element ( $this_layout, 'field', $loop_ID );

      }

    }
  }

  if ( $resolve == true ) {

    foreach ( array_reverse ( $GLOBALS['elements']['types'] ) as $type => $data ) {

      if ( $data['is_open'] == true ) {
        close_element ( $type );
      }

    }

  }

}






//
// SET UP ELEMENT
// create an $element array from the given sub-field type
//

function setup_element ( $type, $generator, $acf_ID ) {

  // index of the given $type in the master array

  $index = array_search ( $type , array_keys ( $GLOBALS['elements']['types'] ) );

  //
  // CURRENT ELEMENT ARRAY
  // stores all the data and settings about the element
  //

  $GLOBALS['elements']['current'] = array(
    'type' => $type,
    'bg' => array(),
    // 'id' => get_element_ID ( $type ),
    'classes' => array(),
    'settings' => array()
  );

  //
  // BLOCK ADJUSTMENTS
  // if it's a block there are a bunch of other things to check for
  // before getting settings etc.
  //

  if ( $type == 'block' || $type == 'block_interactive' || $type == 'block_navigation' ) {

    $block_type = '';

    if ( strpos ( $type, '_' ) !== false ) {
      $block_type = explode ( '_', $type )[1];
    }

    $GLOBALS['elements']['current']['type'] = 'block';
    $type = 'block';

    $content_field = get_sub_field ( 'blocks' );

    $GLOBALS['elements']['current']['block'] = array(
      'type' => $block_type,
      'content' => $content_field[0]['acf_fc_layout']
    );

    $GLOBALS['elements']['current']['classes'][] = 'block-type-' . $GLOBALS['elements']['current']['block']['content'];

    if ( $GLOBALS['elements']['current']['block']['content'] == 'jumbotron' ) {
      $GLOBALS['elements']['current']['classes'][] = 'jumbotron jumbotron-fluid';
    }

		if ( $GLOBALS['elements']['current']['block']['content'] == 'card' ) {
      $GLOBALS['elements']['current']['classes'][] = 'card';
    }

  }

  //
  // ATTRIBUTES
  //

  // id

  $the_ID = '';

  if ( get_sub_field ( $type . '_id' ) != '' ) {

    // this element has a custom ID
    $the_ID = strtolower ( str_replace ( ' ', '-', get_sub_field ( $type . '_id' ) ) );

    // set the 'current_id' key of this element type to the custom value
    // this is what will let future elements pick up the parent's custom ID
    // in their own ID strings

    $GLOBALS['elements']['types'][$type]['current_id'] = $the_ID;
  } else {

    // loop through the element types and start building an ID string.
    // if we hit a custom one, replace everything stored so far with the ID

    $break_next = false;
    $i = 0;

    foreach ( $GLOBALS['elements']['types'] as $parent_type => $data ) {

      if ( $break_next == true ) {
        break;
      }

      if ( $data['current_id'] != '' ) {

        $the_ID = $data['current_id'];

      } else {

        if ( $i != 0 ) $the_ID .= '-';
        $the_ID .= $parent_type . '-' . $GLOBALS['elements']['counters'][$parent_type];

      }

      if ( $parent_type == $type ) $break_next = true;

      $i++;

    }

  }

  $GLOBALS['elements']['current']['id'] = $the_ID;

  // classes

  $GLOBALS['elements']['current']['classes'] = array_merge ( $GLOBALS['elements']['current']['classes'], $GLOBALS['elements']['types'][$type]['classes'] );

  if ( $generator == 'auto' ) {
    $GLOBALS['elements']['current']['classes'][] = 'auto-generated';
  } elseif ( get_sub_field ( 'classes' ) != '' ) {
    $GLOBALS['elements']['current']['classes'][] = get_sub_field ( 'classes' );
  }

  // echo $type . ', ' . $GLOBALS['elements']['counters'][$type] .'<br>';

  if ( $GLOBALS['elements']['counters'][$type] == 1 ) {
    $GLOBALS['elements']['current']['classes'][] = 'first-of-type';
  }

  // if there's currently an open carousel

  if (
    isset ( $GLOBALS['vars']['current_carousel'] ) &&
    $GLOBALS['vars']['current_carousel'] != ''
  ) {

    // if this element is one level below the carousel

    $is_carousel = false;

    // go through all the element types

    foreach ( $GLOBALS['elements']['types'] as $element_type => $element_data ) {

      // if the carousel -
      if ( $is_carousel == true && $element_type == $type ) {

        $GLOBALS['elements']['current']['classes'][] = 'swiper-slide';

      }

      //

      if ( $element_type == $GLOBALS['vars']['current_carousel'] ) {

        $is_carousel = true;

      }

    }

  }

  // if there's currently an open tab

  if (
    isset ( $GLOBALS['vars']['current_tabs'] ) &&
    $GLOBALS['vars']['current_tabs'] != ''
  ) {

    if ( $type == $GLOBALS['elements']['types'][$GLOBALS['vars']['current_tabs']]['tabs']['panel'] ) {
        $GLOBALS['elements']['current']['classes'][] = 'tab-panel';
    }

  }

  //
  // SETTINGS
  //

  // DEFAULTS

  // spacing

  $GLOBALS['elements']['current']['settings'] = $GLOBALS['defaults'][$type];

  //
  // ELEMENT DEFAULTS
  // if the element has defaults, merge them with the settings that were pulled from the element field
  //

  if ( is_array ( get_element_defaults ( $GLOBALS['elements']['current'] ) ) ) {

    $GLOBALS['elements']['current']['settings'] = array_merge ( $GLOBALS['defaults'][$type], get_element_defaults ( $GLOBALS['elements']['current'] ) );

  }

  //
  // SETTINGS FIELDS
  // get settings that have been added manually
  // and override the $element['settings'] keys that have just been set
  //

  if ( $generator != 'auto' && have_rows ( 'settings', $acf_ID ) ) {
    while ( have_rows ( 'settings', $acf_ID ) ) {
      the_row();

      switch ( get_row_layout() ) {

        case 'background' :

          $GLOBALS['elements']['current']['bg'] = array (
            'type' => get_sub_field ( 'type' ),
            'colour' => get_sub_field ( 'background' ),
            'image' => get_sub_field ( 'image' ),
            'video' => get_sub_field ( 'video' ),
            'opacity' => get_sub_field ( 'opacity' ),
            'position' => get_sub_field ( 'position' ),
            'attachment' => get_sub_field ( 'attachment' ),
            'size' => get_sub_field ( 'size' )
          );

          $GLOBALS['elements']['current']['classes'][] = 'has-bg';

          if (
            get_sub_field ( 'colour' ) != '' &&
            get_sub_field ( 'colour' ) != 'inherit'
          ) {
            $GLOBALS['elements']['current']['classes'][] = 'bg-' . get_sub_field ( 'colour' );

            // if there's a colour & opacity set, but no image or video
            // add the bg-opacity-x class

            if (
              get_sub_field ( 'opacity' ) != '' &&
              get_sub_field ( 'opacity' ) != 100 &&
              (
                (
                  get_sub_field ( 'type' ) == 'image' &&
                  get_sub_field ( 'image' ) == ''
                ) || (
                  get_sub_field ( 'type' ) == 'video' &&
                  get_sub_field ( 'video' ) == ''
                )
              )
            ) {

              $GLOBALS['elements']['current']['classes'][] = 'bg-opacity-' . get_sub_field ( 'opacity' );

            }

          }

          break;

        case 'flexbox' :
          $GLOBALS['elements']['current']['classes'][] = 'd-flex';

          if ( get_sub_field ( 'direction' ) != '' ) {
            $GLOBALS['elements']['current']['classes'][] = 'flex-' . get_sub_field ( 'direction' );
          }

          if ( get_sub_field ( 'align' ) != '' ) {
            $GLOBALS['elements']['current']['classes'][] = 'align-items-' . get_sub_field ( 'align' );
          }

          if ( get_sub_field ( 'justify' ) != '' ) {
            $GLOBALS['elements']['current']['classes'][] = 'justify-content-' . get_sub_field ( 'justify' );
          }

          break;

        case 'alignment' :
          $GLOBALS['elements']['current']['classes'][] = 'text-' . get_sub_field ( 'alignment' );
          break;

        case 'colours' :

          $GLOBALS['css'] .= generate_css ( get_sub_field ( 'colours' ), '#' . $GLOBALS['elements']['current']['id'] );

          break;

        case 'spacing' :

          $GLOBALS['elements']['current']['settings']['margin_top'] = get_sub_field ( 'margin_top' );
          $GLOBALS['elements']['current']['settings']['margin_bottom'] = get_sub_field ( 'margin_bottom' );
          $GLOBALS['elements']['current']['settings']['padding_top'] = get_sub_field ( 'padding_top' );
          $GLOBALS['elements']['current']['settings']['padding_bottom'] = get_sub_field ( 'padding_bottom' );

          break;

        case 'sticky' :

          if ( get_sub_field ( 'enabled' ) == 1 ) {

            $GLOBALS['elements']['current']['classes'][] = 'sticky';

            if ( get_sub_field ( 'parent' ) != '' ) {
              $GLOBALS['elements']['current']['atts']['sticky-parent'] = get_sub_field ( 'parent' );
            } else {
              $GLOBALS['elements']['current']['atts']['sticky-parent'] = 'body';
            }

            $GLOBALS['elements']['current']['atts']['sticky-offset'] = get_sub_field ( 'offset' );

            $GLOBALS['elements']['current']['atts']['sticky-breakpoint'] = get_sub_field ( 'breakpoint' );

          }

          break;

        case 'collapsible' :

          if ( get_sub_field ( 'enabled' ) == 1 ) {

            $GLOBALS['elements']['types'][$type]['collapsible'] = collapsible_setup();

          }

          break;

        case 'carousel' :
        case 'swiper' :

          // wp_enqueue_script ( 'slick' );

          wp_enqueue_script ( 'swiper' );

          $GLOBALS['elements']['current']['classes'][] = 'carousel-container';

          $GLOBALS['elements']['types'][$type]['carousel'] = carousel_setup ( get_current_element_ID() );

          break;

        case 'tabs' :

          wp_enqueue_script ( 'jquery-ui-core' );
          wp_enqueue_script ( 'jquery-ui-widget' );
          wp_enqueue_script ( 'jquery-effects-core' );
          wp_enqueue_script ( 'jquery-ui-tabs' );

          $GLOBALS['elements']['current']['classes'][] = 'tab-container';
          $GLOBALS['elements']['current']['classes'][] = 'renderable';

          $GLOBALS['elements']['current']['atts']['renderable-type'] = 'tabs';

          $GLOBALS['elements']['types'][$type]['tabs'] = tabs_setup ();

      }

    }
  }

  // spacing

  if ( $GLOBALS['elements']['current']['settings']['margin_top'] != 0 || $GLOBALS['elements']['current']['settings']['margin_bottom'] != 0 ) {
    if ( $GLOBALS['elements']['current']['settings']['margin_top'] == $GLOBALS['elements']['current']['settings']['margin_bottom'] ) {
      $GLOBALS['elements']['current']['classes'][] = 'my-' . $GLOBALS['elements']['current']['settings']['margin_top'];
    } else {
      $GLOBALS['elements']['current']['classes'][] = 'mt-' . $GLOBALS['elements']['current']['settings']['margin_top'];
      $GLOBALS['elements']['current']['classes'][] = 'mb-' . $GLOBALS['elements']['current']['settings']['margin_bottom'];
    }
  }

  if ( $GLOBALS['elements']['current']['settings']['padding_top'] != 0 || $GLOBALS['elements']['current']['settings']['padding_bottom'] != 0 ) {

    if ( $GLOBALS['elements']['current']['settings']['padding_top'] == $GLOBALS['elements']['current']['settings']['padding_bottom'] ) {
      $GLOBALS['elements']['current']['classes'][] = 'py-' . $GLOBALS['elements']['current']['settings']['padding_top'];
    } else {
      $GLOBALS['elements']['current']['classes'][] = 'pt-' . $GLOBALS['elements']['current']['settings']['padding_top'];
      $GLOBALS['elements']['current']['classes'][] = 'pb-' . $GLOBALS['elements']['current']['settings']['padding_bottom'];
    }

  }

  // colours

  if ( isset ( $GLOBALS['elements']['current']['settings']['colours'] ) ) {

    // $GLOBALS['css'] .= generate_css ( $GLOBALS['elements']['current']['settings']['colours'], '#' . $GLOBALS['elements']['current']['id'] );

  }

  //
  // ADJUSTMENTS FOR SPECIFIC TYPES
  //

  // SECTION

  if ( $GLOBALS['elements']['current']['type'] == 'section' ) {

		// if the loop ID is not equal to the current post ID,
		// this is probably a template

		if ( $acf_ID != get_the_ID() ) {
			$GLOBALS['elements']['current']['classes'][] = 'template-' . $acf_ID;
		}

  }

  // CONTAINER

  if ( $GLOBALS['elements']['current']['type'] == 'container' ) {

    // DEFAULTS

    $GLOBALS['elements']['current']['settings']['width'] = $GLOBALS['defaults']['container']['width'];
    $GLOBALS['elements']['current']['settings']['row_classes'] = $GLOBALS['defaults']['container']['width'];

    if ( get_sub_field ( 'width' ) != '' ) {
      $GLOBALS['elements']['current']['settings']['width'] = get_sub_field ( 'width' );
    }

    if ( $GLOBALS['elements']['current']['settings']['width'] != 'full' ) {
      $GLOBALS['elements']['current']['classes'][] = 'has-max-width';
    }

  }

  // COLUMN

  if ( $GLOBALS['elements']['current']['type'] == 'column' ) {

    $breakpoints = get_breakpoints ( get_sub_field ( 'breakpoints' ) );

    if ( !empty ( $breakpoints ) ) {
      $GLOBALS['elements']['current']['classes'] = array_merge ( $GLOBALS['elements']['current']['classes'], $breakpoints );
    } else {
      $GLOBALS['elements']['current']['classes'][] = 'col';
    }

  }

  // BLOCK

  if ( $GLOBALS['elements']['current']['type'] == 'block' ) {

    $block_content = get_sub_field ( 'blocks' );

    $block_type = $block_content[0]['acf_fc_layout'];

    // $GLOBALS['elements']['current']['classes'][] = 'block-type-' . $block_type;

    /*if ( $block_type == 'jumbotron' ) {
      $GLOBALS['elements']['current']['classes'][] = 'jumbotron jumbotron-fluid';
    }*/


  }

  // echo "\n\n" . '<!--' . "\n\n";
  // print_r($element);
  // echo "\n\n" . '-->' . "\n\n";

  return $GLOBALS['elements']['current'];

}

//
// OPEN ELEMENT
// output the opening tag of an element
//

function open_element ( $type, $generator, $acf_ID ) {

  //
  // SETUP ELEMENT
  // create the $element array that stores the ID, classes and other settings
  //

  $GLOBALS['elements']['current'] = setup_element ( $type, $generator, $acf_ID );

  // index of the given $type in the master array

  $index = array_search ( $GLOBALS['elements']['current']['type'], array_keys ( $GLOBALS['elements']['types'] ) );

  //
  // BEGIN OUTPUT
  //

  echo "\n";

	// $tabs = "";

  // for ( $i = 1; $i <= $index; $i += 1 ) {
  //   $tabs .= "\t";
  // }

	// echo "\n\n" .  $GLOBALS['vars']['indent'] . "\n\n" . '<br>' . "\n\n";

  echo str_repeat ( "\t", $GLOBALS['vars']['indent'] );

	$GLOBALS['vars']['indent']++;

	echo '<div id="' . $GLOBALS['elements']['current']['id'] . '"';

  echo ' class="' . implode ( ' ', $GLOBALS['elements']['current']['classes'] ) . '"';

  if ( !empty ( $GLOBALS['elements']['current']['atts'] ) ) {
    foreach ( $GLOBALS['elements']['current']['atts'] as $att => $val ) {

      echo ' data-' . $att . '="' . $val . '"';

    }
  }

  echo '>';

  // BACKGROUND

  output_bg ( $GLOBALS['elements']['current']['bg'] );

  // COLLAPSIBLE

  collapsible_output ( 'open', $type );

  // for ( $i = 0; $i <= $index; $i += 1 ) {
  //   echo "\t";
  // }



  // SECTION

  section_output ( 'open' );

  // CONTAINER

  container_output ( 'open' );

  // CAROUSEL

  carousel_output ( 'open', $type );

  // TABS

  tabs_output ( 'open', $type );

  // BLOCK

  block_output ( 'open' );

  // set the element to 'open'

  $GLOBALS['elements']['types'][$GLOBALS['elements']['current']['type']]['is_open'] = true;

}

//
// CLOSE ELEMENT
//

function close_element ( $type ) {

  if ( strpos ( $type, 'block_' ) !== false ) {
    $type = 'block';
  }

  $index = array_search ( $type , array_keys ( $GLOBALS['elements']['types'] ) );

  // COLLAPSIBLE

  collapsible_output ( 'close', $type );

  // CAROUSEL

  carousel_output ( 'close', $type );

  // CONTAINER

  if ( $type == 'container' ) {
    container_output ( 'close' );
  }

  // SECTION

  if ( $type == 'section' ) {
    section_output ( 'close' );
  }

  // BLOCK

  if ( $type == 'block' ) {
    block_output ( 'close' );
  }

  // ELEMENT

  echo "\n";

  // for ( $i = 1; $i <= $index; $i += 1 ) {
  //   echo "\t";
  // }

	$GLOBALS['vars']['indent']--;

  echo str_repeat ( "\t", $GLOBALS['vars']['indent'] );

	// echo "\n\n" .  $GLOBALS['vars']['indent'] . "\n\n" . '<br>' . "\n\n";

  echo '</div><!-- ' . $type . ' -->';

  // TABS

  tabs_output ( 'close', $type );

  // set this element type to closed

  $GLOBALS['elements']['types'][$type]['is_open'] = false;
  $GLOBALS['elements']['current'] = null;

  //
  // ID STUFF
  //

  // unset the 'current id' value of this element type
  // since it won't be used for any more children

  $GLOBALS['elements']['types'][$type]['current_id'] = '';

  // pop the last element off the global ID array

  $remove_ID = array_pop ( $GLOBALS['elements']['id'] );

  //
  // COUNTERS
  // reset the counters for each type below this one
  // i.e. if closing a container, reset the counter for column and block to 1
  //

  foreach ( array_reverse ( $GLOBALS['elements']['types'] ) as $element_type => $data ) {

    if ( $element_type != $type ) {

      // this is a child of the element we're closing,
      // so reset it to 1

      $GLOBALS['elements']['counters'][$element_type] = 1;

    } elseif ( $element_type == $type ) {

      // increment the counter of the element we're closing by 1
      $GLOBALS['elements']['counters'][$element_type]++;

      // found what we needed, stop now
      break;

    }
  }

}

//
// RESOLVE ELEMENTS
// given an element type, go through the element_types array and resolve open/closed statuses
// first go backwards and close elements under the current one
// then go forwards and open new elements as necessary
//

function resolve_elements ( $this_layout, $acf_ID = null ) {

	if ( $acf_ID == null ) {
		$acf_ID = get_the_ID();
	}

  if ( strpos ( $this_layout, 'block_' ) !== false ) {
    $this_layout = 'block';
  }

  // if this element type is open,
  // we need to close this and everyone BELOW it
  // i.e. if this is a container, make sure block, column, and container are closed

  // go backwards through $GLOBALS['elements']['types'] until we get to this type
  // and close it if necessary

  $array_index = count ( $GLOBALS['elements']['types'] ) - 1;

  foreach ( array_reverse ( $GLOBALS['elements']['types'] ) as $type => $data ) {

    if ( $type != $this_layout && $data['is_open'] == true ) {

      close_element ( $type );

    } elseif ( $type == $this_layout ) {

      // we got to the element type that matches the field
      // if it's open, then it's either the same type as the previous field
      // or it's a parent that was skipped over in the layout

      // i.e. a new container after a block
      // block and column have been closed already, now close container

      if ( $data['is_open'] == true ) {
        close_element ( $type );
      }

      break;

    }

    $array_index--;

  }

  // now go forwards through $GLOBALS['elements']['types'] and open new elements

  $array_index = 0;

  foreach ( $GLOBALS['elements']['types'] as $type => $data ) {

    if ( $type != $this_layout && $data['is_open'] == false ) {

      open_element ( $type, 'auto', $acf_ID );

    } elseif ( $type == $this_layout ) {

      break;

    }

    $array_index++;

  }

}
