// post grid
// dependencies: jquery
// v1.0

(function ($) {

    /**
     * custom select class
      */
    function post_grid(item, options) {
        this.item = $(item);

        // options
        let defaults = {
            append: false,
            classes: {
                item_wrap: 'query-item-wrap',
                item: 'query-item',
                next_btn: 'post-grid-next-btn'
            },
            query_args: {},
            elements: {
                next_btn: null,
                filter_menu: null,
                filter_toggle: null
            },
            debug: false
        };

        this.options = $.extend(true, defaults, options);

        this.init();
    }

    post_grid.prototype = {

        // init
        init: function () {
            var plugin_instance = this;
            var plugin_item = plugin_instance.item;
            var plugin_settings = plugin_instance.options;
            var plugin_classes = plugin_settings.classes;
            var plugin_elements = plugin_settings.elements;

            if (plugin_settings.debug === true) {
                console.log('post grid', 'init');
            }

            //
            // FEATURES
            //


            //
            // ELEMENTS
            //
            if (plugin_item.find('.' + plugin_classes.next_btn).length) {
                plugin_elements.next_btn = plugin_item.find('.' + plugin_classes.next_btn)
            }

            // item wrapper
            plugin_elements.item_wrap = plugin_item.find('.' + plugin_classes.item_wrap)

            // filter menu
            if (plugin_item.find('.query-filter-wrap').length) {
                plugin_elements.filter_menu = plugin_item.find('.query-filter-wrap')
                plugin_elements.filter_toggle = plugin_item.find('.query-filter-toggle')

                if (
                    plugin_item.hasClass('filter-position-left') &&
                    plugin_item.hasClass('filter-collapsible')
                ) {
                    console.log(sticky_offset)

                    plugin_elements.filter_menu.stick_in_parent({
                        parent: '#' + plugin_item.attr('id'),
                        offset_top: sticky_offset
                    }).on("sticky_kit:bottom", function (e) {
                        $(this).addClass('bottomed')
                    }).on("sticky_kit:unbottom", function (e) {
                        $(this).removeClass('bottomed')
                    })

                    plugin_elements.filter_toggle.stick_in_parent({
                        parent: '#' + plugin_item.attr('id'),
                        offset_top: sticky_offset
                    }).on("sticky_kit:bottom", function (e) {
                        $(this).addClass('bottomed')
                    }).on("sticky_kit:unbottom", function (e) {
                        $(this).removeClass('bottomed')
                    })
                }
            } else if (
                plugin_item.hasClass('filter-position-top') &&
                plugin_item.hasClass('filter-collapsible')
            ) {
            }

            //
            // INIT
            //

            //
            // ACTIONS
            //

            // console.log(plugin_elements.filter_menu)

            if (plugin_elements.filter_menu != null) {
                // FILTER TOGGLE CLICK
                plugin_elements.filter_toggle.find('.btn').click(function () {
                    if (plugin_item.hasClass('filter-open')) {
                        // open
                        plugin_elements.filter_toggle.find('.icon').removeClass('fa-times').addClass('ellipsis-v')
                        plugin_item.removeClass('filter-open')

                        if (plugin_item.hasClass('filter-position-top')) {
                            plugin_elements.filter_menu.slideUp()
                        }
                    } else {
                        // close
                        plugin_elements.filter_toggle.find('.icon').addClass('fa-times').removeClass('ellipsis-v')
                        plugin_item.addClass('filter-open')

                        if (plugin_item.hasClass('filter-position-top')) {
                            plugin_elements.filter_menu.slideDown()
                        }
                    }
                })

                // FILTER ITEM CLICK
                plugin_elements.filter_menu.find('.query-filter-item').click(function (e) {
                    // de-select if turned on already
                    if ($(this).hasClass('selected')) {
                        $(this).removeClass('selected')
                        $(this).find('.icon').removeClass('fa-square-check').addClass('fa-square')

                    } else {
                        $(this).addClass('selected')
                        $(this).find('.icon').removeClass('fa-square').addClass('fa-square-check')

                        // remove sibling classes if not a multi-selectable filter

                        if ($(this).closest('.query-filter-list').attr('data-multi') === 'false') {
                            $(this).siblings().each(function () {
                                $(this).removeClass('selected')
                                $(this).find('.icon').removeClass('fa-square-check').addClass('fa-square')
                            })
                        }
                    }
                    plugin_instance._eval_filters()

                })

                // CLEAR FILTERS
                plugin_elements.filter_menu.find('.query-filter-clear').click(function (e) {
                    plugin_elements.filter_menu.find('.selected').each(function () {
                        $(this).removeClass('selected')
                        $(this).find('.icon').removeClass('fa-square-check').addClass('fa-square')
                    })

                    plugin_instance._eval_filters()
                })

            }

            // NEXT PAGE BUTTON
            if (plugin_elements.next_btn != null) {
                plugin_elements.next_btn.click(function (e) {
                    e.preventDefault()
                    e.stopImmediatePropagation()

                    var clicked_btn = $(this)
                    var link_href = clicked_btn.attr('href')

                    plugin_settings.append = true

                    plugin_elements.item_wrap.addClass('loading')
                    $('body').addClass('spinner-on')

                    plugin_item.acf_query('get_page', {
                        url: link_href,
                        args: plugin_settings.query_args,
                        success: function (success_data) {
                            plugin_instance._add_items({
                                success_data: success_data
                            })
                        },
                        complete: function () {
                            plugin_elements.item_wrap.removeClass('loading')
                            $('body').removeClass('spinner-on')

                            setTimeout(function () {
                                $(document.body).trigger('sticky_kit:recalc');
                            }, 200)
                        }
                    });
                })
            }

            if (plugin_settings.debug === true) {
                console.log('post grid', 'initialized')
            }
        },

        _eval_filters: function (fn_options) {
            var plugin_instance = this;
            var plugin_item = this.item;
            var plugin_settings = plugin_instance.options;
            var plugin_classes = plugin_settings.classes;
            var plugin_elements = plugin_settings.elements;

            // options

            var defaults = {
                clicked: null,
                append: false
            };

            var settings = $.extend(true, defaults, fn_options)

            if (plugin_elements.filter_menu.find('.selected').length) {
                plugin_elements.filter_menu.find('.query-filter-clear').fadeIn()
            } else {
                plugin_elements.filter_menu.find('.query-filter-clear').fadeOut()
            }

            plugin_item.acf_query('eval_filters', {
                complete: function () {

                    console.log('eval done, get page now')

                    plugin_settings.append = false

                    plugin_settings.query_args[plugin_item.closest('.h7-block').attr('id') + '_filters'] = plugin_item.attr('data-query-filters')

                    plugin_elements.item_wrap.addClass('loading')
                    $('body').addClass('spinner-on')

                    console.log(plugin_settings.query_args)

                    plugin_item.acf_query('get_page', {
                        url: window.location.href,
                        args: plugin_settings.query_args,
                        success: function (success_data) {

                            plugin_instance._add_items({
                                success_data: success_data
                            })

                        },
                        complete: function () {

                            plugin_elements.item_wrap.removeClass('loading')
                            $('body').removeClass('spinner-on')

                            setTimeout(function () {
                                $(document.body).trigger('sticky_kit:recalc');
                            }, 200)
                        }
                    })
                }
            })
        },

        _add_items: function (fn_options) {
            var plugin_instance = this;
            var plugin_item = this.item;
            var plugin_settings = plugin_instance.options;
            var plugin_classes = plugin_settings.classes;
            var plugin_elements = plugin_settings.elements;

            // options

            var settings = $.extend(true, {
                success_data: null,
                append: false
            }, fn_options);

            // console.log('success data', settings.success_data)

            if (settings.success_data != null) {
                if (plugin_settings.append === false) {
                    // remove existing items
                    plugin_elements.item_wrap.find('.query-item').remove()
                }

                // add new items
                settings.success_data.items.appendTo(plugin_elements.item_wrap)

                // fire plugins
                $('[data-slick]').each(function () {
                    if (!$(this).hasClass('slick-initialized')) {
                        $(this).slick();
                    }
                });

                // console.log(success_data.next_link)

                if (settings.success_data.next_link === false) {
                    if (plugin_elements.next_btn != null) {
                        plugin_elements.next_btn.fadeOut(250, function () {
                            // $(this).remove()
                        })
                    }
                } else {
                    if (plugin_settings.debug === true) {
                        console.log('change next btn link href')
                    }

                    if (plugin_elements.next_btn != null) {
                        plugin_elements.next_btn.show()

                        plugin_elements.next_btn.attr('href', settings.success_data.next_link)
                    }
                }
            }
        }
    }

    // jQuery plugin interface
    $.fn.post_grid = function (opt) {
        var args = Array.prototype.slice.call(arguments, 1);

        return this.each(function () {
            var item = $(this);
            var instance = item.data('post_grid');

            if (!instance) {
                // create plugin instance if not created
                item.data('post_grid', new post_grid(this, opt));
            } else {
                // otherwise check arguments for method call
                if (typeof opt === 'string') {
                    instance[opt].apply(instance, args);
                }
            }
        });
    }
}(jQuery));
