// swiper
// v1.0
// dependencies: jquery UI slider

(function ($) {

  // custom select class

  function swiper(item, options) {

    this.item = $(item);

    // options

    var defaults = {
      layout: 'horizontal',
      handle: '<div class="swiper-handle rounded-circle bg-primary text-white">',
      icon: '<i class="fas fa-sort fa-rotate-90"></i></div>',
      is_mousedown: false,
      is_dragging: false,
      left_with_mousedown: false,
      animation_speed: 125,
      elements: {},
      debug: false
    };

    this.options = $.extend(true, defaults, options);
    this.init();
  }

  swiper.prototype = {

    // init

    init: function () {

      var plugin_instance = this;
      var plugin_item = plugin_instance.item;
      var plugin_settings = plugin_instance.options;
      var plugin_elements = plugin_settings.elements;

      //
      // INIT
      //

      if (plugin_settings.debug == true) {
        console.log('swiper');
      }

      plugin_item.addClass('swiper-initialized swiper-wrap')

      // remove fallback content
      plugin_item.empty()

      // adjust for vertical

      if (
        typeof plugin_item.attr('data-swiper-layout') !== 'undefined' &&
        (
          plugin_item.attr('data-swiper-layout') == 'horizontal' ||
          plugin_item.attr('data-swiper-layout') == 'vertical'
        )
      ) {

        plugin_settings.layout = plugin_item.attr('data-swiper-layout')

      }

      if (plugin_settings.layout == 'vertical') {
        plugin_settings.icon = '<i class="fas fa-sort"></i></div>'
      }

      //
      // ELEMENTS
      //

      plugin_settings.before_img = plugin_item.attr('data-swiper-before')
      plugin_settings.after_img = plugin_item.attr('data-swiper-after')

      plugin_elements.before = $('<div class="swiper-img swiper-before" style="background-image: url(' + plugin_settings.before_img + ')">').appendTo(plugin_item)

      plugin_elements.after = $('<img src="' + plugin_settings.after_img + '">').appendTo(plugin_item)

      plugin_elements.cover = $('<div class="swiper-cover">').appendTo(plugin_item)

      plugin_elements.handle = $(plugin_settings.handle + plugin_settings.icon).appendTo(plugin_item)

      //
      // ACTIONS
      //

      // click on image

      // drag

      plugin_item.on('mousedown', function(e) {
        e.preventDefault();
        plugin_settings.is_mousedown = true
        plugin_item.addClass('mousedown')

      })

      plugin_item.on('mouseup', function(e) {

        var calc_offset = e.offsetX

        if (plugin_settings.layout == 'vertical') {
          calc_offset = e.offsetY
        }

        plugin_instance._calculate({
          offset: calc_offset
        });

        plugin_settings.is_mousedown = false
        plugin_settings.is_dragging = false
        plugin_item.removeClass('mousedown dragging')

      })

      plugin_item.on('mousemove', function(e) {

        if (plugin_settings.is_mousedown == true) {

          var calc_offset = e.offsetX

          if (plugin_settings.layout == 'vertical') {
            calc_offset = e.offsetY
          }

          plugin_settings.is_dragging = true
          plugin_item.addClass('dragging')

          plugin_instance._calculate({
            offset: calc_offset
          })

        }

      })

      plugin_item.on('mouseleave', function(e) {

        if (plugin_settings.is_mousedown == true) {

          e.preventDefault();

          var calc_offset = e.offsetX

          if (plugin_settings.layout == 'vertical') {
            calc_offset = e.offsetY
          }

          plugin_instance._calculate({
            offset: calc_offset
          });

          plugin_settings.left_with_mousedown = true

          $(document).on('mouseup', function() {
            console.log('left with mouse down, then mouse up')
            plugin_settings.left_with_mousedown = false
            plugin_settings.is_mousedown = false
            plugin_item.removeClass('mousedown')
          })

        }

        plugin_settings.is_dragging = false
        plugin_item.removeClass('dragging')

      })

      plugin_item.on('mouseenter', function(e) {

        plugin_settings.left_with_mousedown

        if (plugin_settings.left_with_mousedown == true) {

          var calc_offset = e.offsetX

          if (plugin_settings.layout == 'vertical') {
            calc_offset = e.offsetY
          }

          plugin_instance._calculate({
            offset: calc_offset
          });

          plugin_settings.is_mousedown = true
          plugin_settings.is_dragging = true
          plugin_item.addClass('mousedown dragging')

        }

      })

      if (plugin_settings.debug == true) {
        console.log('swiper', 'initialized');
      }

    },

    _calculate: function(fn_options) {

      var plugin_instance = this
      var plugin_item = this.item
      var plugin_settings = plugin_instance.options
      var plugin_elements = plugin_settings.elements

      // function settings

      var settings = $.extend(true, {
        offset: null
      }, fn_options)

      if (plugin_settings.debug == true) {
        // console.log('swiper', 'calculating')
      }

      var before_calc = (settings.offset / plugin_item.outerWidth()) * 100

      if (plugin_settings.layout == 'vertical') {
        before_calc = (settings.offset / plugin_item.outerHeight()) * 100
      }

      if (before_calc > 100) {
        before_calc = '100%'
      } else if (settings.offset < 4) {
        before_calc = '4px'
      } else {
        before_calc += '%'
      }

      var before_prop = 'width',
          handle_prop = 'left'

      if (plugin_settings.layout == 'vertical') {
        before_prop = 'height'
        handle_prop = 'top'
      }

      if (plugin_settings.is_dragging == true) {

        plugin_elements.before.css(before_prop, before_calc)
        plugin_elements.handle.css(handle_prop, before_calc)

      } else {

        var before_anim = {},
            handle_anim = {}

        before_anim[before_prop] = before_calc
        handle_anim[handle_prop] = before_calc

        plugin_elements.before.animate(before_anim, plugin_settings.animation_speed)

        plugin_elements.handle.animate(handle_anim, plugin_settings.animation_speed)

      }



    }

  }

  // jQuery plugin interface

  $.fn.swiper = function (opt) {
    var args = Array.prototype.slice.call(arguments, 1);

    return this.each(function () {

      var item = $(this);
      var instance = item.data('swiper');

      if (!instance) {

        // create plugin instance if not created
        item.data('swiper', new swiper(this, opt));

      } else {

        // otherwise check arguments for method call
        if (typeof opt === 'string') {
          instance[opt].apply(instance, args);
        }

      }
    });
  }

}(jQuery));
